module baseRun                                       ! Complete runs of BASE

! Routines:
! 
! ini                    Performs initialisations
! parseOpts              Parses options from config file and command line
! evalDtProps            Evaluates data properties
! prepAnalysis           Prepares the analysis (called once)
! prepNPlMd              Prepares pars. for a #-planets mode
! updtRuntm              Updates max. runtime in cur. #-pl. mode

  use dataModelsNoise
  use inference
  implicit none

!–––––––––––––––––––––––––––––––––––––––––––––––––[ DECLARATIONS AND DEFINITIONS ]––––––––––––––––––––––––––––––––––––––––––––––––––


  !========================================================[ VARIABLES ]============================================================

  integer :: &
   psdNPlReq

  logical :: &
   help, &
   helpParsDQs


!–––––––––––––––––––––––––––––––––––––––––––––––––––––––[ SUBPROGRAM PART ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  contains


!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine ini
!
!   performs initialisations
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     k

    tmStartPrg=omp_get_wtime()
    call date_and_time(values=dateStartPrg)  ! needed by filNmPrefx()
    consLineLen=CONS_LINE_LEN_STD
    consTabStop=CONS_TAB_STOP_STD
    consTabStopOpt=CONS_TAB_STOP_OPT_STD
    call undef(hostName)


    ! config file

    call get_command(cmdLin)
    filNmConf=STD_FIL_NM_CONF
    confFilExists=pathExists(filNmConf)
    if (confFilExists) call tryOpenFilRW("configuration",filNmConf,UN_CONF,stat="old",action="read")


    ! initialisations

    priNPts=0
    dtProps%nSets=0
    call undef(mdlProps%mS)
    call undef(mdlProps%mSinIMax)
    onlyPlotData=.FALSE.
    onlySpdMeas=.FALSE.
    call undef(usrMaxLen)
    call undef(usrMaxLenInMd)
    call undef(usrMaxRuntmS)
    call undef(usrMaxRuntmInMdS)
    keepSeed=.FALSE.
    nPT=2
    maxRuntmRatioDiscard=.1          ! CAUTION: when changing this value, also change corresponding help text
    PSRThrld1Fact=.9_dp              ! CAUTION: when changing this value, also change corresponding help text
    PSRThrld2=1.1_dp
    aftrBurnRatioConv=1.
    aftrBurnRatioNotConv=.25
    PTNChains=8
    meanPTBlkLen=100
    help=.FALSE.
    helpParsDQs=.FALSE.
    filOutput=.TRUE.
    filOutputPlt=.TRUE.
    nThin=1
    tmpNPriRges=0
    call undef(tmpPriTy)
    tmpNCalcProb=0
    tmpNCalcProbAllPars=0
    pltNPts=PLT_N_PTS%std
    pltNPtsMdl=PLT_N_PTS_MDL%std
    pltNPtsScatter=PLT_N_PTS_SCATTER%std
    pltNPtsKDE=PLT_N_PTS_KDE%std
    pred1D%nPts=PLT_N_PTS_PREDICT%std
    pred2D%nPts=PLT_N_PTS_PREDICT%std
    mdlProps%bin=.FALSE.
    call undef(maxNPl)
    call undef(usrSpdMeasDurS)
    call undef(usrTParAdjDurS)
    usrAdjustTPar=.TRUE.
    call undef(infcLenFrac)
    execGnuplt=.TRUE.
    execPS4PDF=.TRUE.
    singlePlts=.FALSE.
    pltPSTricks=.FALSE.
    usrInfcLenMin=0
    usrKMaxFromVel=.FALSE.
    usrScatterPlts=.FALSE.
    call undef(nPeriodsRge)
    dataDir=DATA_DIR_STD
    call undef(outDir)
    call undef(outDirCSV)
    genDtProps%sysNm=GEN_D_NM_STD
    genDtProps%uniTm=.FALSE.
    genDtProps%polCoord(1)=.FALSE.
    genDtProps%nWin=0
    call undef(genDtProps%onlyDtTy)
    call undef(genDtProps%nRec)
    maximiseInfcLen=.FALSE.
    usrJointMaPo=.TRUE.
    usrMaPo=.TRUE.
    usrDQs=.TRUE.
    usrQuiet=.FALSE.
    call undef(usrComment)
    call undef(transTmStart)
    nUsrHPDI=0
    nHPDI=N_STD_HPDI
    HPDIProbCont(1:nHPDI)=STD_HPDI_PROB_CONT(1:nHPDI)
    call undef(HPDIRole)
    HPDIRole(1:nHPDI)=[(k, k=1,nHPDI)]
    iStdHPDI(1:nHPDI)=HPDIRole(1:nHPDI)
    usrNoInfc=.FALSE.
    usrReadSamp=.FALSE.
    usrSavSamp=.FALSE.
    usrSavCandCSV=.FALSE.
    call undef(savSamp%filNmRead)
    call undef(savSamp%filNmSav)
    usrPeriodogram=.FALSE.
    mailAddrFin=""                   ! undef(mailAddrFin) cannot be used because isDef() is not accessible by basicIO module
    call undef(zmRgeAllPars)
    usrZmRge=.FALSE.
    tmpNGroup=0
    call undef(dtProps%sysNm)
    sectDepth=0
    call undef(mdlProps%epoch)
    call undef(genDtProps%epoch)

    call buildCRCTbl                 ! build CRC table for saved posterior


  end subroutine ini




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine parseOpts
!
!   parses options from config file and command line
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j, &
     pos1, &
     pos2, &
     PTNChainsTmp, &
     readStat, &
     allocStat, &
     tmpInt, &
     iOpt, &
     iS1, &
     iS2, &
     k, &
     l, &
     iOptClashWReadSamp, &
     iOptClashWSavSamp, &
     iOptClashWNoInfc, &
     nOccur(N_CL_OPTS), &
     nOccurScp(N_CL_OPTS), &
     posTmp, &
     iDtSet
    real :: &
     tmpReal
    real(dp) :: &
     tmpDbl2(2), &
     tmpDbl(2), &
     tmpDblAlloc(:), &
     tmpD, &
     tmpUsrHPDI(:), &
     tParTmp(:)
    logical :: &
     PTNChainsSet, &
     filNow, &
     scop
    character(STR_LEN_LONGEST) :: &
     cOpt, &
     cOptRem, &
     cTmp
    character(STR_LEN_STD) :: &
     cErrMsg
    character(len(OPT_PREFX)) :: &
     curPrefx
    type(tyIntvlDbl) :: &
     tRgePred
    allocatable :: &
     tParTmp, &
     tmpUsrHPDI, &
     tmpDblAlloc

    call undef(iOptClashWReadSamp)
    call undef(iOptClashWSavSamp)
    call undef(iOptClashWNoInfc)
    call undef(tRgePred)
    PTNChainsSet=.FALSE.

    
    ! get # data files and potential --help option from command line

    i=0
    do
      call inc(i)
      call get_command_argument(i, cOpt)
      if (len_trim(cOpt) == 0) exit
      if (.NOT. beginsWith(cOpt,OPT_PREFX)) then ! argument is data file name
        if (dtProps%nSets == N_DT_SETS_MAX) call err("too many (>"//trim(niceFmt(N_DT_SETS_MAX))//") data sets")
        call inc(dtProps%nSets)
        dtProps%filNms(dtProps%nSets)=cOpt
        cycle
      else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%help)%short))) then
        help=.TRUE.
        return
      end if
    end do
    
    
    ! get other options

    nConfFilLins=0
    iDtSet=0
    if (confFilExists .AND. .NOT. help) then
      filNow=.TRUE.
    else
      filNow=.FALSE.
    end if
    do                               ! loop to read config file and then command line

      i=0
      scop=.FALSE.
      iS1=1
      iS2=N_N_PL_MODES_MAX
      nOccur=0

LOOP_ARGS: &
      do
        call inc(i)

          ! get current option

        if (filNow) then
          curPrefx=""
          read(UN_CONF,"(A)",iostat=readStat,err=999,iomsg=cErrMsg) cOpt
          if (readStat /= 0) exit LOOP_ARGS
          cOpt=adjustl(cOpt)
          if (cOpt == "" .OR. cOpt(1:1) == COMMENT_PREFX) then
            cycle LOOP_ARGS
          else
            call inc(nConfFilLins)
            confFilLins(nConfFilLins)=cOpt
          end if
        else
          curPrefx=OPT_PREFX
          call get_command_argument(i, cOpt)
          if (len_trim(cOpt) == 0) exit LOOP_ARGS
          if (.NOT. beginsWith(cOpt,OPT_PREFX)) then ! argument is data file name
            call inc(iDtSet)
            cycle
          end if
        end if

          ! evaluate option

        iOpt=getOptIndx(cOpt,filNow) ! unique option index
        if (isUndef(iOpt)) call err("unrecognised option: "//trim(cOpt))
        if (iOpt /= I_CL_OPTS%readSamp .AND. .NOT. CL_OPTS(iOpt)%combWReadSamp) iOptClashWReadSamp=iOpt
        if (iOpt /= I_CL_OPTS%savSamp .AND. .NOT. CL_OPTS(iOpt)%combWSavSamp) iOptClashWSavSamp=iOpt
        if (iOpt /= I_CL_OPTS%noInfc .AND. .NOT. CL_OPTS(iOpt)%combWNoInfc) iOptClashWNoInfc=iOpt

        call inc(nOccur(iOpt))
        call inc(nOccurScp(iOpt))
        if (isDef(CL_OPTS(iOpt)%nMax) .AND. nOccur(iOpt) > CL_OPTS(iOpt)%nMax) &
         call err("option '"//trim(CL_OPTS(iOpt)%short)//"' cannot be supplied more than "//trim(mrg("once", &
         mrg("twice",trim(niceFmt(CL_OPTS(iOpt)%nMax))//" times",CL_OPTS(iOpt)%nMax == 2),CL_OPTS(iOpt)%nMax == 1)))

        if (scop .AND. isDef(CL_OPTS(iOpt)%nMaxScp) .AND. nOccurScp(iOpt) > CL_OPTS(iOpt)%nMaxScp) then
          if (CL_OPTS(iOpt)%nMaxScp == 0) then
            call err("option '"//trim(CL_OPTS(iOpt)%short)//"' cannot be used after the "//trim(curPrefx)// &
             trim(CL_OPTS(I_CL_OPTS%scope)%short)//" option")
          else
            call err("option '"//trim(CL_OPTS(iOpt)%short)//"' cannot be supplied more than "//trim(mrg("once", &
             mrg("twice",trim(niceFmt(CL_OPTS(iOpt)%nMax))//" times",CL_OPTS(iOpt)%nMax == 2),CL_OPTS(iOpt)%nMax == 1))// &
             " per scope")
          end if
        end if

        if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%scope)%short),tmpInt)) then
          if (tmpInt <= 0) call err("option '"//trim(CL_OPTS(I_CL_OPTS%scope)%short)// &
           "': index of #-planets mode must be positive")
          nOccurScp=0
          iS1=tmpInt
          iS2=iS1
          scop=.TRUE.

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcHPDI)%short)//"=",pos1,OPT_LONGER)) then
          call xplode(cOpt(pos1:),CL_OPT_LIST_SEP,tmpUsrHPDI)
          nUsrHPDI=size(tmpUsrHPDI,1)
          if (nUsrHPDI > N_USR_HPDI_MAX) &
           call err("at most "//trim(niceFmt(N_USR_HPDI_MAX))//" additional HPDIs can be calculated")
          call undef(HPDIRole)
          HPDIProbCont(nHPDI+1:nHPDI+nUsrHPDI)=tmpUsrHPDI(1:nUsrHPDI)
          nHPDI=nHPDI+nUsrHPDI
          block
            integer(i8b) :: &
             iTmpHPDI(N_STD_HPDI+N_USR_HPDI_MAX), &
             k

            call indxArr(HPDIProbCont(1:nHPDI),iTmpHPDI(1:nHPDI))
            do k=1,N_STD_HPDI
              l=posInArr(iTmpHPDI(1:nHPDI),k)
              iStdHPDI(k)=l
              HPDIRole(l)=k
            end do
            HPDIProbCont(1:nHPDI)=HPDIProbCont(iTmpHPDI(1:nHPDI))

          end block

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcProb)%short)//"[",pos1,OPT_LONGER)) then
          call parseCalcProb

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcProbAllPars)%short)//"=",pos1,OPT_LONGER)) then
          call parseCalcProbAllPars

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%calcTransitAfter)%short),tmpD)) then
          transTmStart=tmpD

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%comment)%short),cTmp)) then
          usrComment=trim(adjustl(cTmp))

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%dataDir)%short),cTmp)) then
          dataDir=trim(adjustl(cTmp))
          tmpInt=len_trim(dataDir)
          if (dataDir(tmpInt:tmpInt) /= "/") dataDir=trim(dataDir)//"/"

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%discardFrac)%short),tmpReal)) then
          if (.NOT. (tmpReal >= 0 .AND. tmpReal < 1)) call err("max. runtime fraction to be discarded must be >= 0 and < 1")
          maxRuntmRatioDiscard(iS1:iS2)=tmpReal

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%discardPSRScale)%short),tmpReal)) then
          if (.NOT. (tmpReal > 0 .AND. tmpReal < 1.)) call err("PSR scale for stopping discarding must be positive and < 1")
          PSRThrld1Fact(iS1:iS2)=tmpReal

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%generateData)%short)//"[",pos1,OPT_LONGER)) then ! needs to be treated specially
          call parseGenDt

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%epoch)%short),tmpD)) then
          mdlProps%epoch=tmpD
          genDtProps%epoch=mdlProps%epoch

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%group)%short)//"[",pos1,OPT_LONGER)) then
          if (filNow) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%group)%short)//" cannot be used in config file")
          if (iDtSet == 0) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%group)%short)//" must follow a file name")
          call inc(tmpNGroup)
          tmpGroupDtSets(tmpNGroup)=iDtSet
          call xplode(rmTrail(trim(cOpt(pos1:)),"]"),",",tmpDblAlloc)
          block
            integer :: &
             s

            s=size(tmpDblAlloc,1)
            if (s > maxval(N_GROUP_THRLDS)) call err("option "//trim(curPrefx)// &
             trim(CL_OPTS(I_CL_OPTS%group)%short)//": wrong format")
            call undef(tmpGroupThrlds(tmpNGroup,:))
            tmpGroupThrlds(tmpNGroup,:s)=tmpDblAlloc

          end block

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%helpParsDQs)%short))) then
          helpParsDQs=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%inferenceLen)%short)//"=max")) then
          maximiseInfcLen=.TRUE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%inferenceLen)%short),tmpInt)) then
          if (tmpInt <= 0) call err("# links used for inference must be positive")
          usrInfcLenMin(iS1:iS2)=tmpInt

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%inferenceLenFrac)%short),tmpReal)) then
          if (tmpReal <= 0) call err("fraction <# links for inference>/<chain length> must be positive")
          infcLenFrac(iS1:iS2)=tmpReal

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%keepSeed)%short))) then
          keepSeed=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%KMaxFromVel)%short))) then
          usrKMaxFromVel(iS1:iS2)=.TRUE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%maxDur)%short),tmpReal,OPT_IS_TIME)) then
          if (scop) then
            usrMaxRuntmInMdS(iS1:iS2)=nint(tmpReal)
          else
            usrMaxRuntmS=nint(tmpReal)
          end if

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%maxLen)%short),tmpInt)) then
          if (tmpInt < TH_INFC_LEN_MIN) call err("maximum number of links must be >= "//trim(niceFmt(TH_INFC_LEN_MIN)))
          if (scop) then
            usrMaxLenInMd(iS1:iS2)=tmpInt
          else
            usrMaxLen=tmpInt
          end if

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%M)%short),mdlProps%mS)) then ! [M_Sun]; rescaled by prepAnalysis() later
          if (mdlProps%mS <= 0) call err("M must be positive")

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%mailWhenFinished)%short),cOpt)) then
          mailAddrFin=cOpt

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%mSinIMax)%short),tmpReal)) then ! [M_J]
          if (tmpReal <= 0) call err("(m*sin(i))_max must be positive")
          mdlProps%mSinIMax(iS1:iS2)=tmpReal

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noInfc)%short))) then
          usrNoInfc=.TRUE.
          usrJointMaPo(iS1:iS2)=.FALSE.
          usrMaPo(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noPlt)%short))) then
          filOutputPlt=.FALSE.
          usrJointMaPo(iS1:iS2)=.FALSE.
          usrMaPo(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noDQs)%short))) then
          usrDQs(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noFiles)%short))) then
          filOutput=.FALSE.
          filOutputPlt=.FALSE.
          usrJointMaPo(iS1:iS2)=.FALSE.
          usrMaPo(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noGnuplotExec)%short))) then
          execGnuplt=.FALSE.
          execPS4PDF=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noJointMaPo)%short))) then
          usrJointMaPo(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noMaPo)%short))) then
          usrMaPo(iS1:iS2)=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noPS4PDFExec)%short))) then
          execPS4PDF=.FALSE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%noTempParsAdj)%short))) then
          usrAdjustTPar(iS1:iS2)=.FALSE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%nPeriodsRange)%short),tmpDblAlloc)) then
          if (size(tmpDblAlloc) < 2 .OR. tmpDblAlloc(1) >= tmpDblAlloc(2)) &
           call err("option '"//trim(CL_OPTS(I_CL_OPTS%nPeriodsRange)%short)//"': wrong format")
          nPeriodsRge(iS1:iS2)=tyIntvlDbl(tmpDblAlloc(1),tmpDblAlloc(2))

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%nPl)%short)//"=",pos1,OPT_LONGER)) then ! needs to be treated specially
          call xplode(cOpt(pos1:),CL_OPT_LIST_SEP,nPl)
          nNPlMds=size(nPl,1)
          if (nNPlMds > N_N_PL_MODES_MAX) call err("maximum number of #-planets modes is "//trim(niceFmt(N_N_PL_MODES_MAX)))
          call sort(nPl(1:nNPlMds))
          maxNPl=nPl(nNPlMds)

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%nPT)%short),nPT)) then
          if (nPT < 1) call err("number of PT procedures must be >= 1")

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%onlyPlotData)%short))) then
          onlyPlotData=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%onlySpeedMeas)%short))) then
          onlySpdMeas=.TRUE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%outDir)%short),cTmp)) then
          outDir=trim(adjustl(cTmp))
          tmpInt=len_trim(outDir)
          if (outDir(tmpInt:tmpInt) /= "/") outDir=trim(outDir)//"/"

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%outDirCSV)%short),cTmp)) then
          outDirCSV=trim(adjustl(cTmp))
          tmpInt=len_trim(outDirCSV)
          if (outDirCSV(tmpInt:tmpInt) /= "/") outDirCSV=trim(outDirCSV)//"/"

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%periodogram)%short))) then
          usrPeriodogram=.TRUE.
!           usrDQs=.FALSE.
          usrJointMaPo=.FALSE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%predict)%short),tmpDblAlloc)) then
          if (size(tmpDblAlloc,1) < 2) call err("option '"//trim(CL_OPTS(I_CL_OPTS%predict)%short)//"': wrong format")
          tRgePred%lBnd=tmpDblAlloc(1)
          tRgePred%uBnd=tmpDblAlloc(2)

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%prior)%short)//"[",pos1,OPT_LONGER)) then
          pos2=scan(cOpt(pos1+1:),"]")+pos1-1
          if (pos2 == pos1-1) call err("option '"//trim(CL_OPTS(I_CL_OPTS%prior)%short)//"': wrong format")

          call inc(tmpNPriFromFil(iS1:iS2))
          tmpPriNmsFromFil(iS1:iS2,tmpNPriFromFil(iS1))=cOpt(pos1:pos2)
          tmpPriFilNms(iS1:iS2,tmpNPriFromFil(iS1))=adjustl(cOpt(pos2+3:))

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%PSRConv)%short),PSRThrld2)) then
          if (PSRThrld2 < 1._dp) call err("PSR threshold for convergence must be >= 1")

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%PSTricksPlots)%short))) then
          pltPSTricks=.TRUE.
          singlePlts=.TRUE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%PTNChains)%short),PTNChainsTmp)) then
          if (PTNChainsTmp < 1) call err("number of parallel chains must be >= 1")
          if (allocated(tParTmp) .AND. PTNChainsTmp /= PTNChains) call errOptClash(I_CL_OPTS%PTNChains,I_CL_OPTS%tempPars)
          PTNChains=PTNChainsTmp
          PTNChainsSet=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%quickPlots)%short))) then
          pltNPts=PLT_N_PTS%quick
          pltNPtsMdl=PLT_N_PTS_MDL%quick
          pltNPtsScatter=PLT_N_PTS_SCATTER%quick
          pltNPtsKDE=PLT_N_PTS_KDE%quick
          pred1D%nPts=PLT_N_PTS_PREDICT%quick
          pred2D%nPts=PLT_N_PTS_PREDICT%quick

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%quiet)%short))) then
          usrQuiet=.TRUE.

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%range)%short)//"[",pos1,OPT_LONGER)) then
          call parseRange

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%readSamp)%short),cTmp)) then
          savSamp%filNmRead=cTmp
          usrReadSamp=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%savCandCSV)%short))) then
          usrSavCandCSV=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%savSamp)%short))) then
          usrSavSamp=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%scatterPlots)%short))) then
          usrScatterPlts(iS1:iS2)=.TRUE.

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%singlePlots)%short))) then
          singlePlts=.TRUE.

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%speedMeasDur)%short),tmpReal,OPT_IS_TIME)) then
          usrSpdMeasDurS(iS1:iS2)=nint(tmpReal)

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%stdOutLineLen)%short),tmpInt)) then
          if (tmpInt < CONS_LINE_LEN_MIN) call err("stdout line length must be >= "//trim(niceFmt(CONS_LINE_LEN_MIN)))
          consLineLen=tmpInt
          consTabStop=max(CONS_TAB_WIDTH+2,min(consLineLen-CONS_VAL_LEN_MIN,CONS_TAB_STOP_STD))
          consTabStopOpt=max(CONS_TAB_WIDTH+2,min(consLineLen-CONS_OPT_LEN_MIN,CONS_TAB_STOP_OPT_STD))

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%swap)%short),tmpInt)) then
          if (tmpInt <= 1) call err("mean spacing between PT swap proposals must be > 1")
          meanPTBlkLen(iS1:iS2)=tmpInt

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%tempPars)%short)//"=",pos1,OPT_LONGER)) then ! needs to be treated specially
          PTNChainsTmp=2+count([(cOpt(j:j), j=pos1+1,len_trim(cOpt))] == CL_OPT_LIST_SEP)
          if (PTNChainsSet .AND. PTNChainsTmp /= PTNChains) call errOptClash(I_CL_OPTS%PTNChains,I_CL_OPTS%tempPars)
          PTNChains=PTNChainsTmp
          call xplode(trim(niceFmt(TPAR_MAX))//CL_OPT_LIST_SEP//trim(cOpt(pos1:)),CL_OPT_LIST_SEP,tParTmp)
          call sort(tParTmp,OPT_SORT_DESCENDING)

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%tempParsAdjDur)%short),tmpReal,OPT_IS_TIME)) then
          usrTParAdjDurS(iS1:iS2)=nint(tmpReal)

        else if (tryOptCmd(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%thin)%short)//"=auto")) then
          call undef(nThin(iS1:iS2))

        else if (tryOptAssign(filNow,trim(cOpt),trim(CL_OPTS(I_CL_OPTS%thin)%short),tmpInt)) then
          if (tmpInt < 1) call err("stride for thinning must be >= 1")
          nThin(iS1:iS2)=tmpInt

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%value)%short)//"[",pos1,OPT_LONGER)) then
          call parseVal

        else if (beginsWith(trim(cOpt),trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%zmRgeAllPars)%short)//"=",pos1,OPT_LONGER)) then
          call parseZmRgeAllPars

        else
          call err("unrecognised option: "//trim(cOpt))

        end if

      end do LOOP_ARGS

      if (filNow) then
        filNow=.FALSE.
      else
        exit
      end if

    end do

    
    ! check for clashing opts.
    
    if (usrReadSamp) then
      if (usrSavSamp) then
        call warnOptClash(I_CL_OPTS%savSamp,I_CL_OPTS%readSamp)
        usrSavSamp=.FALSE.
      else if (isDef(iOptClashWReadSamp)) then
        call warnOptClash(iOptClashWReadSamp,I_CL_OPTS%readSamp)
      end if
    end if
    
    if (isDef(iOptClashWSavSamp) .AND. usrSavSamp) call warnOptClash(iOptClashWSavSamp,I_CL_OPTS%savSamp)
    if (isDef(iOptClashWNoInfc) .AND. usrNoInfc) call warnOptClash(iOptClashWNoInfc,I_CL_OPTS%noInfc)
    if (usrNoInfc .AND. .NOT. usrSavSamp) call warn("samples are not saved and no inference will be made from them")

    if (usrPeriodogram) then
      if (.NOT. all(usrMaPo)) call err("marginal posteriors cannot be deactivated in periodogram mode")
      if (pltNPts == PLT_N_PTS%quick) call errOptClash(I_CL_OPTS%quickPlots,I_CL_OPTS%periodogram)
    end if
    if (.NOT. filOutputPlt .AND. onlyPlotData) call err("deactivating file output not allowed when using option "// &
     trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%onlyPlotData)%short))


    if (any(isDef(mdlProps%mSinIMax) .AND. usrKMaxFromVel)) call errOptClash(I_CL_OPTS%mSinIMax,I_CL_OPTS%KMaxFromVel)
    where (isUndef(mdlProps%mSinIMax)) mdlProps%mSinIMax=PLANET_MASS_MAX_PHYS
    
    if (usrSavCandCSV) then
      if (onlySpdMeas) call errOptClash(I_CL_OPTS%savCandCSV,I_CL_OPTS%onlySpeedMeas)
      if (.NOT. filOutput) call errOptClash(I_CL_OPTS%savCandCSV,I_CL_OPTS%noFiles)
    end if

    
    if (PTNChains == 1 .AND. any(usrAdjustTPar)) then
      call warn("adjustment of tempering parameters has been deactivated as there are no hot chains")
      usrAdjustTPar=.FALSE.
    end if

    if (all(isUndef(infcLenFrac)) .AND. .NOT. usrReadSamp) infcLenFrac=INFC_LEN_FRAC_STD

    if (usrReadSamp .AND. any(isUndef(nThin))) call err("automatic thinning impossible when reading in samples")

    if (execGnuplt .AND. .NOT. pltPSTricks .AND. .NOT. execPS4PDF) call err("option "//trim(curPrefx)// &
     trim(CL_OPTS(I_CL_OPTS%noPS4PDFExec)%short)//" can only be used in combination with "//trim(curPrefx)// &
     trim(CL_OPTS(I_CL_OPTS%PSTricksPlots)%short))

    execPS4PDF=execPS4PDF .AND. pltPSTricks

    if (genDtProps%nWin > 0 .AND. .NOT. usrGenerateData) &
     call err("time-windows options can only be given when generating data")

    if (.NOT. (usrGenerateData .OR. help .OR. helpParsDQs .OR. usrReadSamp) .AND. dtProps%nSets == 0) call err("no data given")
    if (usrGenerateData .AND. dtProps%nSets /= genDtProps%nSets) call err("number of file names for generating data must be "// &
     trim(niceFmt(genDtProps%nSets)))

    if (onlySpdMeas) then
      if (isDef(usrMaxLen) .OR. isDef(usrMaxRuntmS) .OR. any(isDef(usrMaxRuntmInMdS) .OR. isDef(usrMaxLenInMd))) &
       call err("setting maximum runtime or length when only performing speed measurement is invalid")
      if (isUndef(usrSpdMeasDurS(1))) call err("duration of speed measurement not set")
    end if

    
    ! set tempering pars.
    
    if (allocated(tParTmp)) then
      tParMin=tParTmp(PTNChains)
    else
      tParMin=STD_TPAR_MIN
      allocate(tParTmp(PTNChains),stat=allocStat)
      call testAllocErr(allocStat,14)
      if (PTNChains == 1) then
        tParTmp(1)=TPAR_MAX
      else
        do i=1,PTNChains
          tParTmp(i)=TPAR_MAX-real(i-1,dp)/(PTNChains-1)*(TPAR_MAX-tParMin)
        end do
      end if
    end if
    allocate(tPar(nPT,PTNChains),tParSumOfSq(nPT),tParAdjRemS(nPT),stat=allocStat)
    call testAllocErr(allocStat,15)
    do i=1,nPT
      tPar(i,:)=tParTmp(:)
    end do
    tParMeanSqDevnMax=(.1_dp*(TPAR_MAX-tParMin))**2


    pltFilExt=mrg(SUFFX_TEX,SUFFX_PS,pltPSTricks)

    call setIfUndef(outDir,OUT_DIR_STD)
    call setIfUndef(outDirCSV,trim(outDir)//SUBDIR_CSV_STD)
    if (.NOT. (help .OR. helpParsDQs) .AND. .NOT. pathExists(outDir)) call err("output directory does not exist: "//trim(outDir))
    if (.NOT. (help .OR. helpParsDQs) .AND. .NOT. pathExists(outDirCSV)) call err("CSV output directory does not exist: "// &
     trim(outDirCSV))

    if (isDef(tRgePred)) then

      allocate(pred1D%best(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred1D%nPts),pred1D%unc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred1D%nPts), &
       pred1D%minUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred1D%maxUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred1D%tMinUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred1D%tMaxUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred1D%t(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred1D%nPts),pred1D%tResol(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred1D%tRge(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred1D%skipSet(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred2D%best(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred2D%nPts),pred2D%unc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred2D%nPts), &
       pred2D%minUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred2D%maxUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred2D%phiMinUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred2D%phiMaxUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred2D%tMinUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred2D%tMaxUnc(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred2D%t(N_DT_TYPES,N_DT_SETS_PER_TY_MAX,pred2D%nPts),pred2D%tResol(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       pred2D%tRge(N_DT_TYPES,N_DT_SETS_PER_TY_MAX),pred2D%skipSet(N_DT_TYPES,N_DT_SETS_PER_TY_MAX), &
       stat=allocStat)
      call testAllocErr(allocStat,18)

      pred1D%tRge=tRgePred
      pred2D%tRge=tRgePred
      usrPred=.TRUE.
    else
      usrPred=.FALSE.
    end if


    if (confFilExists) close(UN_CONF)

    return

999 call err("error reading config file ("//trim(cErrMsg)//")")


    contains

    subroutine errOptClash(i,j)      ! reports clashing options; casts error
      implicit none
      integer :: &
       i, &
       j
      intent(in) :: &
       i, &
       j

      call err("clashing options: "//trim(CL_OPTS(i)%short)//", "//trim(CL_OPTS(j)%short))
    end subroutine


    subroutine warnOptClash(i,j)     ! reports clashing options; produces warning
      implicit none
      integer :: &
       i, &
       j
      intent(in) :: &
       i, &
       j

      call warn("ignoring option "//trim(CL_OPTS(i)%short)//" as it clashes with option "//trim(CL_OPTS(j)%short))
    end subroutine


    subroutine parseVal              ! parses string for a single value
      implicit none
      character(STR_LEN_STD) :: &
       tmpStr
      type(tyIntvlDbl) :: &
       rge

      pos2=scan(cOpt(pos1+1:),"]")+pos1-1
      if (pos2 == pos1-1) call err("option '"//trim(CL_OPTS(I_CL_OPTS%value)%short)//"': wrong format")
      tmpStr=cOpt(pos1:pos2)

      pos1=pos2+3
      read(cOpt(pos1:),*) rge%lBnd
      rge%uBnd=rge%lBnd

      call inc(tmpNPriRges(iS1:iS2))

      tmpPriRgeNms(iS1:iS2,tmpNPriRges(iS1))=tmpStr
      tmpPriRge(iS1:iS2,tmpNPriRges(iS1))=rge
    end subroutine


    subroutine parseRange            ! parses string for a range
      implicit none
      integer :: &
       priTy
      real(dp) :: &
       rgeArr(:), &
       vceN
      character(STR_LEN_STD) :: &
       parNm
      type(tyIntvlDbl) :: &
       rge
      allocatable :: &
       rgeArr

      call undef(vceN)
      pos2=scan(cOpt(pos1+1:),"]")+pos1-1
      if (pos2 == pos1-1) call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)//"]': wrong format")
      parNm=cOpt(pos1:pos2)
      
      pos1=pos2+3
      if (cOpt(pos1:pos1) == "[") then
        call undef(priTy)
        call inc(pos1)
      else                           ! user-defined prior type (one of the default ones)
        priTy=posInArr(PRI_TY_ID,cOpt(pos1:pos1))-1 ! indexing of PRI_TY_ID is zero-based
        if (isUndef(priTy) .OR. priTy == PRI_TY%fil) call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)// &
         "]': wrong format")
        call inc(pos1)

        if (priTy == PRI_TY%norm) then
          if (cOpt(pos1:pos1) /= "[") call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)// &
           "]': wrong format")
          call inc(pos1)
          pos2=scan(cOpt(pos1+1:),"]")+pos1-1
          vceN=dblFromStr(cOpt(pos1:pos2))
          pos1=pos2+2
        end if

        if (cOpt(pos1:pos1) /= ":") call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)//"]': wrong format")
        pos1=pos1+2
      end if
      pos2=scan(cOpt(pos1+1:),"]")+pos1-1
      
      call xplode(cOpt(pos1:pos2),CL_OPT_LIST_SEP,rgeArr)
      if (size(rgeArr,1) /= 2) call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)//"]': wrong format")
      rge%lBnd=rgeArr(1)
      rge%uBnd=rgeArr(2)
      if (rge%width() < 0) call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)//"]': invalid range")
      
      call inc(tmpNPriRges(iS1:iS2))

      tmpPriRgeNms(iS1:iS2,tmpNPriRges(iS1))=parNm
      tmpPriRge(iS1:iS2,tmpNPriRges(iS1))=rge
      tmpPriTy(iS1:iS2,tmpNPriRges(iS1))=priTy
      if (isDef(vceN)) tmpPriVce(iS1:iS2,tmpNPriRges(iS1))=(.5_dp*rge%width()/vceN)**2
    end subroutine


    subroutine parseZmRgeAllPars     ! parses string for zoom range of all pars.
      implicit none
      integer :: &
       i, &
       pos3
      logical :: &
       curOmit, &
       fin

      fin=.FALSE.
      do i=1,N_L_PARS_MAX
        pos2=scan(cOpt(pos1+1:),CL_OPT_LIST_SEP)+pos1
        curOmit=pos2 == pos1+1
        pos3=scan(cOpt(pos2+1:),CL_OPT_LIST_SEP)+pos2
        if (pos3 == pos2) then
          fin=.TRUE.
          pos3=len_trim(cOpt)+1
        end if
        if (curOmit .AND. pos3-pos2 > 1) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%zmRgeAllPars)%short)// &
         ": wrong format")
        if (.NOT. curOmit) then
          zmRgeAllPars(iS1:iS2,i)=tyIntvlDbl(dblFromStr(cOpt(pos1+1:pos2-1)),dblFromStr(cOpt(pos2+1:pos3-1)))
          if (zmRgeAllPars(iS1,i)%width() <= 0) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%zmRgeAllPars)%short)// &
           ": sides of hypercube must have positive lengths")
        end if
        if (fin) exit
        if (i == N_L_PARS_MAX) call err("option '"//trim(CL_OPTS(I_CL_OPTS%zmRgeAllPars)%short)//"': wrong format")
        pos1=pos3
      end do
      usrZmRge(iS1:iS2)=.TRUE.
    end subroutine


    subroutine parseCalcProb         ! parses string for interval over which to calc. posterior prob.
      implicit none
      real(dp) :: &
       rgeArr(:)
      character(STR_LEN_STD) :: &
       parNm
      type(tyIntvlDbl) :: &
       rge
      allocatable :: &
       rgeArr

      pos2=scan(cOpt(pos1+1:),"]")+pos1-1
      if (pos2 == pos1-1) call err("option '"//trim(CL_OPTS(I_CL_OPTS%calcProb)%short)//"': wrong format")
      parNm=cOpt(pos1:pos2)

      pos1=pos2+4
      pos2=scan(cOpt(pos1+1:),"]")+pos1-1

      call xplode(cOpt(pos1:pos2),CL_OPT_LIST_SEP,rgeArr)
      if (size(rgeArr,1) /= 2) call err("option '"//trim(CL_OPTS(I_CL_OPTS%calcProb)%short)//"["//trim(parNm)//"]': wrong format")
      rge%lBnd=rgeArr(1)
      rge%uBnd=rgeArr(2)
      if (rge%width() < 0) call err("option '"//trim(CL_OPTS(I_CL_OPTS%range)%short)//"["//trim(parNm)//"]': invalid range")

      call inc(tmpNCalcProb(iS1:iS2))

      tmpCalcProbNms(iS1:iS2,tmpNCalcProb(iS1))=parNm
      tmpCalcProbRge(iS1:iS2,tmpNCalcProb(iS1))=rge
    end subroutine


    subroutine parseCalcProbAllPars  ! parses string for hypercube over which to calc. posterior prob.
      implicit none
      integer :: &
       i, &
       pos3
      logical :: &
       curOmit, &
       fin

      call inc(tmpNCalcProbAllPars(iS1:iS2))
      call dec(pos1)
      call undef(tmpCalcProbAllParsRegion(iS1:iS2,tmpNCalcProbAllPars(iS1),:))

      fin=.FALSE.
      do i=1,N_L_PARS_MAX
        pos2=scan(cOpt(pos1+1:),CL_OPT_LIST_SEP)+pos1
        curOmit=pos2 == pos1+1
        pos3=scan(cOpt(pos2+1:),CL_OPT_LIST_SEP)+pos2
        if (pos3 == pos2) then
          fin=.TRUE.
          pos3=len_trim(cOpt)+1
        end if
        if (curOmit .AND. pos3-pos2 > 1) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcProbAllPars)%short)// &
         ": wrong format")
        if (.NOT. curOmit) then
          tmpCalcProbAllParsRegion(iS1:iS2,tmpNCalcProbAllPars(iS1),i)= &
           tyIntvlDbl(dblFromStr(cOpt(pos1+1:pos2-1)),dblFromStr(cOpt(pos2+1:pos3-1)))
          if (tmpCalcProbAllParsRegion(iS1,tmpNCalcProbAllPars(iS1),i)%width() <= 0) &
           call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcProbAllPars)%short)// &
           ": sides of hypercube must have positive lengths")
        end if
        if (fin) exit
        if (i == N_L_PARS_MAX) call err("option "//trim(curPrefx)//trim(CL_OPTS(I_CL_OPTS%calcProbAllPars)%short)// &
         ": wrong format")
        pos1=pos3
      end do

      tmpCalcProbAllParsDim(iS1:iS2,tmpNCalcProbAllPars(iS1))=i
    end subroutine


    subroutine parseGenDt            ! parses string for data-generation options
      implicit none
      integer :: &
       iPar
      real(dp) :: &
       paramsArr(:)
      allocatable :: &
       paramsArr


      cOptRem=cOpt(pos1:)
      pos1=1
      usrGenerateData=.TRUE.
      filOutput=.FALSE.



      ! data type to generate

      pos2=scan(cOptRem,CL_OPT_LIST_SEP)-1
      genDtProps%dtTy=posInArr(DT_TY_CMDS,cOptRem(pos1:pos2))

      genDtProps%nSetsOfTy=0
      genDtProps%onlyDtTy=genDtProps%dtTy
      if (isUndef(genDtProps%dtTy)) call err("data type '"//cOptRem(pos1:pos2)//"' invalid")
      genDtMdlProps%bin=genDtProps%dtTy == DT_TY%AMAngPos


      ! command(s)

      tmpInt=0
      pos1=pos2+2
      do
        pos2=pos1
        do
          pos2=scan(cOptRem(pos2+1:),CL_OPT_LIST_SEP)+pos2-1
          cTmp=cOptRem(pos1:pos2)
          if (scan(cTmp,"[") == 0 .OR. (scan(cTmp,"[") > 0 .AND. scan(cTmp,"]") > 0)) exit
          pos2=pos2+2
        end do
        if (scan(cTmp,CMD_PREFX) > 0) then ! command(s) supplied
          call inc(tmpInt)
          if (tmpInt > GEN_D_N_USR_CMDS_MAX) call err("number of commands for generating data must be <= "// &
           trim(niceFmt(GEN_D_N_USR_CMDS_MAX)))

          ! parse command
          if (trim(cTmp) == CMD_PREFX//DT_CMDS%polCoord) then
            if (genDtProps%dtTy /= DT_TY%AMAngPos) call err("command "//CMD_PREFX//trim(DT_CMDS%polCoord)// &
              " can only be given when generating "//trim(DT_TY_DESC(DT_TY%AMAngPos)))
            genDtProps%polCoord(1)=.TRUE.
          elseif (trim(cTmp) == CMD_PREFX//trim(DT_CMDS%nBObs)//"2") then
            if (genDtProps%dtTy /= DT_TY%RV) call err("command "//CMD_PREFX//trim(DT_CMDS%nBObs)//"2"// &
              " can only be given when generating "//trim(DT_TY_DESC(DT_TY%RV)))
            genDtMdlProps%bin=.TRUE.
          elseif (trim(cTmp) == CMD_PREFX//DT_CMDS%withAMAngPos) then
            if (genDtProps%dtTy /= DT_TY%RV) call err("command "//CMD_PREFX//trim(DT_CMDS%withAMAngPos)// &
              " can only be given when generating "//trim(DT_TY_DESC(DT_TY%RV)))
            call undef(genDtProps%onlyDtTy)
            genDtProps%nSetsOfTy(DT_TY%AMAngPos)=1
            genDtMdlProps%bin=.TRUE.
          elseif (trim(cTmp) == CMD_PREFX//DT_CMDS%withRV) then
            if (genDtProps%dtTy /= DT_TY%AMAngPos) call err("command "//CMD_PREFX//trim(DT_CMDS%withRV)// &
              " can only be given when generating "//trim(DT_TY_DESC(DT_TY%AMAngPos)))
            call undef(genDtProps%onlyDtTy)
            genDtProps%nSetsOfTy(DT_TY%RV)=1
          elseif (beginsWith(cTmp,trim(CMD_PREFX//DT_CMDS%window)//"=",posTmp)) then
            call inc(genDtProps%nWin)
            if (genDtProps%nWin > GEN_D_N_WINDOWS_MAX) call err("number of time windows must be <= "// &
             trim(niceFmt(GEN_D_N_WINDOWS_MAX)))
            call xplode(rmTrail(trim(cTmp(posTmp+1:)),"]"),CL_OPT_LIST_SEP,tmpDblAlloc)
            if (size(tmpDblAlloc,1) < 2) call err("assignment of '"//trim(DT_CMDS%window)//"': wrong format")
            genDtProps%win(genDtProps%nWin)=tmpDblAlloc(1)
            genDtProps%winFrac(genDtProps%nWin)=tmpDblAlloc(2)
          elseif (beginsWith(cTmp,trim(CMD_PREFX//DT_CMDS%times)//"=",posTmp)) then
            call xplode(rmTrail(trim(cTmp(posTmp+1:)),"]"),CL_OPT_LIST_SEP,tmpDblAlloc)
            genDtProps%nRec=size(tmpDblAlloc,1)
            if (allocated(genDtProps%times)) call err(" command "//CMD_PREFX//trim(DT_CMDS%times)// &
             " cannot be given more than once")
            allocate(genDtProps%times(size(tmpDblAlloc)),stat=allocStat)
            call testAllocErr(allocStat,75)
            genDtProps%times=tmpDblAlloc
          elseif (trim(cTmp) == CMD_PREFX//DT_CMDS%uniTm) then
            genDtProps%uniTm=.TRUE.
          else
            call err("invalid command for generating data: "//trim(cTmp))
          end if

          pos1=pos2+2
        else
          exit
        end if
      end do
      
      if (genDtProps%nWin > 0 .AND. genDtProps%uniTm) call err("combining commands "//CMD_PREFX//trim(DT_CMDS%window)//" and "// &
       CMD_PREFX//trim(DT_CMDS%uniTm)//" not allowed")
      if (allocated(genDtProps%times)) then
        if (genDtProps%nWin > 0) call err("combining commands "//CMD_PREFX//trim(DT_CMDS%times)//" and "// &
         CMD_PREFX//trim(DT_CMDS%window)//" not allowed")
        if (genDtProps%uniTm) call err("combining commands "//CMD_PREFX//trim(DT_CMDS%times)//" and "// &
         CMD_PREFX//trim(DT_CMDS%uniTm)//" not allowed")
      end if


      mdlProps%bin=genDtMdlProps%bin
      call xplode(rmTrail(trim(cOptRem(pos1:)),"]"),CL_OPT_LIST_SEP,paramsArr)
      iPar=1


      ! # pars., (# planets)

      select case (genDtProps%dtTy)
      case (DT_TY%AMAngPos)
        genDtProps%nSets=1
        genDtProps%nSetsOfTy(DT_TY%AMAngPos)=genDtProps%nSets
      case (DT_TY%RV)
        if (genDtMdlProps%bin) then
          genDtProps%nSets=2
          genDtProps%nSetsOfTy(DT_TY%RV)=genDtProps%nSets
        else
          genDtPsdNPl=paramsArr(iPar)
          call inc(iPar)
          pos1=pos1+2
          genDtProps%nSets=1
          genDtProps%nSetsOfTy(DT_TY%RV)=genDtProps%nSets
        end if
      end select

      if (genDtMdlProps%bin) genDtPsdNPl=1

      call setParsDQs(genDtMdlProps%bin,genDtProps,genDtPsdNPl,OPT_NO_DQS,genDtNPars)


      ! pars. etc.

      allocate(genDtPars(genDtNPars),stat=allocStat)
      call testAllocErr(allocStat,16)

      genDtNSgm=nSubStr(cOptRem(pos1:),CL_OPT_LIST_SEP)-genDtNPars-merge(2,3,genDtMdlProps%bin)
      if (isDef(genDtProps%nRec) .AND. .NOT. ((genDtProps%dtTy == DT_TY%AMAngPos .AND. genDtNSgm/2 == genDtProps%nRec) .OR. &
       (genDtProps%dtTy /= DT_TY%AMAngPos .AND. genDtNSgm == genDtProps%nRec))) &
       call err("number of data inconsistently specified (# uncertainties must match # data)")

      if (.NOT.((genDtMdlProps%bin .AND. ((genDtProps%dtTy == DT_TY%AMAngPos .AND. genDtNSgm >= 2 .AND. isEven(genDtNSgm)) &
       .OR. (genDtProps%dtTy /= DT_TY%AMAngPos .AND. genDtNSgm >= 1))) .OR. (.NOT. genDtMdlProps%bin .AND. genDtNSgm >= 1))) &
       call err("generate-data: wrong number of arguments (expected number of model parameters: "//trim(niceFmt(genDtNPars))//")")
      
      genDtPars(1:genDtNPars)=paramsArr(iPar:iPar+genDtNPars-1)
      iPar=iPar+genDtNPars
      if (.NOT. genDtMdlProps%bin) then
        genDtMdlProps%mS=paramsArr(iPar)
        call inc(iPar)
      end if
      if (isDef(genDtProps%nRec) .AND. paramsArr(iPar) /= genDtProps%nRec) &
       call err("number of data inconsistently specified (# data must match # observing times)")
      genDtProps%nRec=paramsArr(iPar)
      call inc(iPar)
      genDtProps%tRge%lBnd=paramsArr(iPar)
      call inc(iPar)
      genDtProps%tRge%uBnd=paramsArr(iPar)
      call inc(iPar)

      if (genDtProps%dtTy == DT_TY%AMAngPos) then
        genDtNSgm=genDtNSgm/2
        allocate(genDtUncEll(genDtNSgm),stat=allocStat)
        call testAllocErr(allocStat,25)
        do j=1,genDtNSgm
          genDtUncEll(j)%a=paramsArr(iPar)
          call inc(iPar)
          genDtUncEll(j)%b=paramsArr(iPar)
          call inc(iPar)
          if (genDtUncEll(j)%a <= 0 .OR. genDtUncEll(j)%b <= 0) call err(trim(TTL_PART%uncEllAx)//" must be positive")
          if (genDtUncEll(j)%a < genDtUncEll(j)%b) call err("semi-major axis of uncertainty ellipse must be >= semi-minor axis")
        end do
      else
        allocate(genDtSgm(genDtNSgm),stat=allocStat)
        call testAllocErr(allocStat,17)
        do j=1,genDtNSgm
          genDtSgm(j)=paramsArr(iPar)
          call inc(iPar)
          if (genDtSgm(j) <= 0) call err("radial-velocity uncertainties must be positive")
        end do
      end if
    end subroutine

  end subroutine parseOpts




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine evalDtProps
!
!   evaluates data properties
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     curNRec, &
     i, &
     iSet, &
     iTmp
    real(dp) :: &
     curT
    logical :: &
     AMDataOkCompo, &
     AMDataOkRefPt


    ! CHECK VALIDITY OF DATA

    if (dtProps%nRec == 0) call err("data files contain no records")


    ! binary components / # observed bodies
    ! if the binary component referred to is undefined for any data set, it must be undefined for all
    ! (data are then treated as referring to one star only; undefined not allowed for AMAngPos data)
    if (any(isUndef(dtProps%binCompo(:dtProps%nSets))) .AND. .NOT. all (isUndef(dtProps%binCompo(:dtProps%nSets)))) &
     call err("binary-component information is set for some but not all data sets")
    dtProps%nBObs=merge(1,2,allEq(dtProps%binCompo(:dtProps%nSets))) ! includes possibility that all components are undefined
    dtProps%nBObsWDtTy(DT_TY%RV)=merge(0,merge(1,2,allEq(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%binCompo)), &
     dtProps%nSetsOfTy(DT_TY%RV) == 0)
      ! includes possibility that all components are undefined

    ! only external reference point(s) used?
    dtProps%onlyExtRef=.NOT.any(dtProps%refIsCompa(:dtProps%nSets))

    ! binary mode: used only if 2 bodies observed or companion used as reference point for some data
    mdlProps%bin=dtProps%nBObs == 2 .OR. .NOT. dtProps%onlyExtRef


    ! only AM/only RV data?
    dtProps%onlyRV=all(dtProps%dtTyOfSets(:dtProps%nSets) == DT_TY%RV)
    call undef(dtProps%onlyDtTy)
    do i=1,N_DT_TYPES
      if (all(dtProps%dtTyOfSets(:dtProps%nSets) == i)) then
        dtProps%onlyDtTy=i
        exit
      end if
    end do


    ! ENFORCE CURRENT RESTRICTIONS CONCERNING AM DATA


    ! current restrictions concerning AM data:
    !  – normal mode; or binary mode with:
    !    * binary component = 2 or undefined (treated as = 2)  (=> n_b,obs=1)
    !    * ref. pt. = companion

    ! any AM data? (only then perform rest of checks)
    if (dtProps%onlyRV) then
      dtProps%nBObsWDtTy(DT_TY%AMAngPos)=0
    else if (mdlProps%bin) then
      ! binary component = (2 or undefined) and ref. pt. = companion for all AM data?
      AMDataOkCompo=.TRUE.
      AMDataOkRefPt=.TRUE.
      do iSet=1,dtProps%nSetsOfTy(DT_TY%AMAngPos)
        AMDataOkCompo=AMDataOKCompo .AND. dtSetsAMAngPos(iSet)%binCompo == 2
        AMDataOkRefPt=AMDataOkRefPt .AND. dtSetsAMAngPos(iSet)%refIsCompa
      end do
      if (.NOT. AMDataOkCompo) call err("analysing "//trim(DT_TY_DESC(DT_TY%AMAngPos))// &
       " of primary component not implemented in binary mode")
      if (.NOT. AMDataOkRefPt) call err("analysing "//trim(DT_TY_DESC(DT_TY%AMAngPos))// &
       " using external reference point(s) not implemented in binary mode")
      dtProps%nBObsWDtTy(DT_TY%AMAngPos)=1
    else
      dtProps%nBObsWDtTy(DT_TY%AMHipN)=1
      dtProps%nBObsWDtTy(DT_TY%RV)=1
    end if



    ! ENFORCE CURRENT RESTRICTIONS CONCERNING RV DATA


    ! current restrictions concerning RV data:
    !  – if binary mode: 2 bodies observed with RV (case 1 in [04.04.2010/7])

    if (dtProps%nSetsOfTy(DT_TY%RV) > 0 .AND. mdlProps%bin .AND. &
     (all(isUndef(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%binCompo)) .OR. allEq(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%binCompo))) &
     call err("analysing "//trim(DT_TY_DESC(DT_TY%RV))//" in binary mode with only one binary component observed not implemented")




    ! ENFORCE CURRENT RESTRICTIONS CONCERNING ALL DATA


    ! current restrictions concerning all data:
    ! (none)




    ! binary mode vs. stellar mass
    if (.NOT. (onlyPlotData .OR. all(.NOT. usrDQs)) .AND. (mdlProps%bin .EQV. isDef(mdlProps%mS))) &
     call err("stellar mass must be provided if derived quantities are to be calculated in normal mode")




    ! DETERMINE SOME ADDITIONAL DATA PROPERTIES


    ! collect all timestamps (without sorting) in uniqT
    ! AMAngPos data: transformations

    call undef(uniqT)                ! array
    nUniqT=0

    ! AMAngPos data
    do iSet=1,dtProps%nSetsOfTy(DT_TY%AMAngPos)
      do i=1,dtSetsAMAngPos(iSet)%N

        ! timestamps
        curT=dtSetsAMAngPos(iSet)%t(i)
        dtPropsAMAngPos%tRge%lBnd=min(dtPropsAMAngPos%tRge%lBnd,curT)
        dtPropsAMAngPos%tRge%uBnd=max(dtPropsAMAngPos%tRge%uBnd,curT)
        iTmp=posInArr(uniqT(1:nUniqT),curT)
        if (isUndef(iTmp)) then      ! not found
          nUniqT=nUniqT+1
          uniqT(nUniqT)=curT
          dtSetsAMAngPos(iSet)%iUniqT(i)=nUniqT
        else                         ! already in array
          dtSetsAMAngPos(iSet)%iUniqT(i)=iTmp
        end if

      end do
    end do
    dtSetsAMAngPos(iSet)%pol=.FALSE. ! transformation to rectangular coords. has been made if necessary

    ! RV data: timestamps
    do iSet=1,dtProps%nSetsOfTy(DT_TY%RV)
      do i=1,dtSetsRV(iSet)%N
        curT=dtSetsRV(iSet)%t(i)
        dtPropsRV%tRge%lBnd=min(dtPropsRV%tRge%lBnd,curT)
        dtPropsRV%tRge%uBnd=max(dtPropsRV%tRge%uBnd,curT)
        iTmp=posInArr(uniqT(1:nUniqT),curT)
        if (isUndef(iTmp)) then      ! not found
          nUniqT=nUniqT+1
          uniqT(nUniqT)=curT
          dtSetsRV(iSet)%iUniqT(i)=nUniqT
        else                         ! already in array
          dtSetsRV(iSet)%iUniqT(i)=iTmp
        end if
      end do
    end do

    ! AMHipN data
    do iSet=1,dtProps%nSetsOfTy(DT_TY%AMHipN)
      do i=1,dtSetsAMHipN(iSet)%N
        curT=dtSetsAMHipN(iSet)%t(i)
        dtPropsAMHipN%tRge%lBnd=min(dtPropsAMHipN%tRge%lBnd,curT)
        dtPropsAMHipN%tRge%uBnd=max(dtPropsAMHipN%tRge%uBnd,curT)
        iTmp=posInArr(uniqT(1:nUniqT),curT)
        if (isUndef(iTmp)) then      ! not found
          nUniqT=nUniqT+1
          uniqT(nUniqT)=curT
          dtSetsAMHipN(iSet)%iUniqT(i)=nUniqT
        else                         ! already in array
          dtSetsAMHipN(iSet)%iUniqT(i)=iTmp
        end if
      end do
    end do

    dtProps%tRge=tyIntvlDbl(minval(uniqT(:nUniqT)),maxval(uniqT(:nUniqT)))
    call setIfUndef(mdlProps%epoch,intvlCentre(dtProps%tRge))
    dtProps%timesAreJD=dtProps%tRge%lBnd >= RJD_OFS

    if (dtProps%timesAreJD) then
      dateDescAndUnit=DATE_DESC_AND_UNIT_GENERIC
      dateDescAndUnitPlt=DATE_DESC_AND_UNIT_GENERIC_SHORT
      tOfs=-RJD_OFS
    else
      if (dtProps%nSetsOfTy(DT_TY%AMHipN) > 0) call err("times must be Julian Dates when using "//trim(DT_TY_DESC(DT_TY%AMHipN))// &
       " data")                      ! since we transform AMHipN times into JD
      dateDescAndUnit=DATE_DESC_AND_UNIT_GENERIC
      dateDescAndUnitPlt=DATE_DESC_AND_UNIT_GENERIC_SHORT
      tOfs=0
    end if

    uniqTMnEpoch(:nUniqT)=uniqT(:nUniqT)-mdlProps%epoch

    ! calc. some data-related quantities

    likhFact=1._dp

    ! AMAngPos data
    dtPropsAMAngPos%obsStdDev=0._dp
    do iSet=1,dtProps%nSetsOfTy(DT_TY%AMAngPos)
      curNRec=dtSetsAMAngPos(iSet)%N
      likhFact=likhFact/TWO_PI**curNRec
      dtSetsAMAngPos(iSet)%tRge=tyIntvlDbl(minval(dtSetsAMAngPos(iSet)%t(:curNRec)),maxval(dtSetsAMAngPos(iSet)%t(:curNRec)))
      dtProps%sngTimeSpansRge%lBnd=min(dtProps%sngTimeSpansRge%lBnd,dtSetsAMAngPos(iSet)%tRge%width())
      dtProps%sngTimeSpansRge%uBnd=max(dtProps%sngTimeSpansRge%uBnd,dtSetsAMAngPos(iSet)%tRge%width())
      dtSetsAMAngPos(iSet)%sepRge=tyIntvlDbl(minval(dtSetsAMAngPos(iSet)%sep(:curNRec)), &
       maxval(dtSetsAMAngPos(iSet)%sep(:curNRec)))
      dtPropsAMAngPos%obsStdDev=dtPropsAMAngPos%obsStdDev+arrVar(dtSetsAMAngPos(iSet)%obs(:curNRec)%x,0._dp)+ & ! corresponds to variance of Euclidean distances
       arrVar(dtSetsAMAngPos(iSet)%obs(:curNRec)%y,0._dp)

      dtSetsAMAngPos(iSet)%unc(:curNRec)%sinPhi=sin(dtSetsAMAngPos(iSet)%unc(:curNRec)%phi)
      dtSetsAMAngPos(iSet)%unc(:curNRec)%cosPhi=cos(dtSetsAMAngPos(iSet)%unc(:curNRec)%phi)
      dtSetsAMAngPos(iSet)%unc(:curNRec)%sinSqPhi=dtSetsAMAngPos(iSet)%unc(:curNRec)%sinPhi**2
      dtSetsAMAngPos(iSet)%unc(:curNRec)%cosSqPhi=dtSetsAMAngPos(iSet)%unc(:curNRec)%cosPhi**2
      dtSetsAMAngPos(iSet)%unc(:curNRec)%recprASq=1._dp/dtSetsAMAngPos(iSet)%unc(:curNRec)%a**2
      dtSetsAMAngPos(iSet)%unc(:curNRec)%recprBSq=1._dp/dtSetsAMAngPos(iSet)%unc(:curNRec)%b**2
    end do
    dtPropsAMAngPos%sepRge=tyIntvlDbl(minval(dtSetsAMAngPos(:dtProps%nSetsOfTy(DT_TY%AMAngPos))%sepRge%lBnd), &
     maxval(dtSetsAMAngPos(:dtProps%nSetsOfTy(DT_TY%AMAngPos))%sepRge%uBnd))
    dtPropsAMAngPos%obsStdDev=sqrt(dtPropsAMAngPos%obsStdDev)


    ! RV data
    call undef(dtPropsRV%obsRgeCompo)
    call undef(dtPropsRV%obsRge2SgmCompo)
    call undef(dtPropsRV%obsRge2SgmSect)
    call undef(dtPropsRV%obsDiff2SgmPerSetMax)
    dtPropsRV%obsStdDev=0._dp
    do iSet=1,dtProps%nSetsOfTy(DT_TY%RV)
      curNRec=dtSetsRV(iSet)%N
      likhFact=likhFact/TWO_PI**(.5_dp*curNRec)
      dtSetsRV(iSet)%sgmSq(:curNRec)=dtSetsRV(iSet)%sgm(:curNRec)**2
      dtSetsRV(iSet)%tMnEpoch(:curNRec)=dtSetsRV(iSet)%t(:curNRec)-mdlProps%epoch
      dtSetsRV(iSet)%tRge=tyIntvlDbl(minval(dtSetsRV(iSet)%t(:curNRec)),maxval(dtSetsRV(iSet)%t(:curNRec)))
      dtProps%sngTimeSpansRge%lBnd=min(dtProps%sngTimeSpansRge%lBnd,dtSetsRV(iSet)%tRge%width())
      dtProps%sngTimeSpansRge%uBnd=max(dtProps%sngTimeSpansRge%uBnd,dtSetsRV(iSet)%tRge%width())
      dtSetsRV(iSet)%obsRge=tyIntvlDbl(minval(dtSetsRV(iSet)%obs(:curNRec)),maxval(dtSetsRV(iSet)%obs(:curNRec)))
      if (isDef(dtSetsRV(iSet)%binCompo)) &
       dtPropsRV%obsRgeCompo(dtSetsRV(iSet)%binCompo)=merge(dtSetsRV(iSet)%obsRge, &
       intvlUnion(dtPropsRV%obsRgeCompo(dtSetsRV(iSet)%binCompo),dtSetsRV(iSet)%obsRge), &
       isUndef(dtPropsRV%obsRgeCompo(dtSetsRV(iSet)%binCompo)))
      dtSetsRV(iSet)%obsRge2Sgm=tyIntvlDbl(minval(dtSetsRV(iSet)%obs(:curNRec)-dtSetsRV(iSet)%sgm(:curNRec)), &
       maxval(dtSetsRV(iSet)%obs(:curNRec)+dtSetsRV(iSet)%sgm(:curNRec)))
      if (isDef(dtSetsRV(iSet)%binCompo)) &
       dtPropsRV%obsRge2SgmCompo(dtSetsRV(iSet)%binCompo)=merge(dtSetsRV(iSet)%obsRge2Sgm, &
       intvlUnion(dtPropsRV%obsRge2SgmCompo(dtSetsRV(iSet)%binCompo),dtSetsRV(iSet)%obsRge2Sgm), &
       isUndef(dtPropsRV%obsRge2SgmCompo(dtSetsRV(iSet)%binCompo)))
      if (isUndef(dtPropsRV%obsRge2SgmSect)) then
        dtPropsRV%obsRge2SgmSect=dtSetsRV(iSet)%obsRge2Sgm
      else
        dtPropsRV%obsRge2SgmSect=dtPropsRV%obsRge2SgmSect%sect(dtSetsRV(iSet)%obsRge2Sgm)
      end if
      dtPropsRV%obsDiff2SgmPerSetMax=merge(dtSetsRV(iSet)%obsRge2Sgm%width(), &
       max(dtSetsRV(iSet)%obsRge2Sgm%width(),dtPropsRV%obsDiff2SgmPerSetMax),isUndef(dtPropsRV%obsDiff2SgmPerSetMax))
      dtPropsRV%obsStdDev=dtPropsRV%obsStdDev+arrVar(dtSetsRV(iSet)%obs(:curNRec))
    end do
    dtPropsRV%obsRge=tyIntvlDbl(minval(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%obsRge%lBnd), &
     maxval(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%obsRge%uBnd))
    dtPropsRV%obsRge2Sgm=tyIntvlDbl(minval(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%obsRge2Sgm%lBnd), &
     maxval(dtSetsRV(:dtProps%nSetsOfTy(DT_TY%RV))%obsRge2Sgm%uBnd))
    dtPropsRV%obsStdDev=sqrt(dtPropsRV%obsStdDev)

    ! AMHipN data
    dtPropsAMHipN%obsStdDev=0._dp
    do iSet=1,dtProps%nSetsOfTy(DT_TY%AMHipN)
      curNRec=dtSetsAMHipN(iSet)%N
      likhFact=likhFact/TWO_PI**(.5_dp*curNRec)
      dtSetsAMHipN(iSet)%sgmSq(:curNRec)=dtSetsAMHipN(iSet)%sgm(:curNRec)**2
      dtSetsAMHipN(iSet)%params%cosPsi(:curNRec)=cos(dtSetsAMHipN(iSet)%params%psi(:curNRec))
      dtSetsAMHipN(iSet)%params%sinPsi(:curNRec)=sin(dtSetsAMHipN(iSet)%params%psi(:curNRec))
      dtSetsAMHipN(iSet)%tRge=tyIntvlDbl(minval(dtSetsAMHipN(iSet)%t(:curNRec)),maxval(dtSetsAMHipN(iSet)%t(:curNRec)))
      dtSetsAMHipN(iSet)%params%pos0AtT(:curNRec)%x=dtSetsAMHipN(iSet)%params%pos0%x+(dtSetsAMHipN(iSet)%t(:curNRec)- &
       HIP_N_MEAN_EPOCH)*dtSetsAMHipN(iSet)%params%mu0%x
      dtSetsAMHipN(iSet)%params%pos0AtT(:curNRec)%y=dtSetsAMHipN(iSet)%params%pos0%y+(dtSetsAMHipN(iSet)%t(:curNRec)- &
       HIP_N_MEAN_EPOCH)*dtSetsAMHipN(iSet)%params%mu0%y
      dtSetsAMHipN(iSet)%tMnEpoch(:curNRec)=dtSetsAMHipN(iSet)%t(:curNRec)-mdlProps%epoch
      dtSetsAMHipN(iSet)%obsRge=tyIntvlDbl(minval(dtSetsAMHipN(iSet)%obs(:curNRec)),maxval(dtSetsAMHipN(iSet)%obs(:curNRec)))
      dtPropsAMHipN%obsStdDev=dtPropsAMHipN%obsStdDev+arrVar(dtSetsAMHipN(iSet)%obs(:curNRec))
    end do
    dtPropsAMHipN%obsRge=tyIntvlDbl(minval(dtSetsAMHipN(:dtProps%nSetsOfTy(DT_TY%AMHipN))%obsRge%lBnd), &
     maxval(dtSetsAMHipN(:dtProps%nSetsOfTy(DT_TY%AMHipN))%obsRge%uBnd))
    dtPropsAMHipN%obsStdDev=sqrt(dtPropsAMHipN%obsStdDev)

  end subroutine evalDtProps




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine prepAnalysis
!
!   prepares the analysis (called once)
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j, &
     p, &
     allocStat

    ! properties of global parameters
    do i=1,N_G_PARS
      gStdPriRge(i)=G_STD_PRI_RGE(i)%sect(G_STRICT_PRI_RGE(i))
    end do

    ! star mass: [M_J] internally
    if (isDef(mdlProps%mS)) mdlProps%mS=mdlProps%mS*M_SUN_IN_MJUP


    DQSkipped=.FALSE.
    if (.NOT. usrReadSamp) call undef(fracIndep)
    call undef(fracIndepMin)
    maPoShft=0
    call undef(priVce)


    ! predictions

    if (usrPred) then

      call undef(pred1D%best)
      call undef(pred1D%unc)
      call undef(pred1D%t)
      call undef(pred1D%minUnc)
      call undef(pred1D%maxUnc)
      call undef(pred1D%tMinUnc)
      call undef(pred1D%tMaxUnc)
      pred1D%tResol=pred1D%tRge(1,1)%width()/(pred1D%nPts-1)
      pred1D%skipSet=.FALSE.
      call undef(pred2D%best)
      call undef(pred2D%unc)
      call undef(pred2D%t)
      call undef(pred2D%phiMinUnc)
      call undef(pred2D%phiMaxUnc)
      call undef(pred2D%minUnc)
      call undef(pred2D%maxUnc)
      call undef(pred2D%tMinUnc)
      call undef(pred2D%tMaxUnc)
      pred2D%skipSet=.FALSE.
      pred2D%tResol=pred2D%tRge(1,1)%width()/(pred2D%nPts-1)

    end if

    if (mdlProps%bin) then           ! BINARY MODE

      ! currently, binary mode implies:
      !  – AM:
      !    * 1 obs. body: component #2 (secondary)
      !    * ref.pt. is companion
      !  – RV:
      !    * 2 obs. bodies (both components)

      ! #-planets modes

      if (isDef(maxNPl)) then
        if (maxNPl > 0) call err("including planets not allowed in binary mode")
      else
        nNPlMds=1
        allocate(nPl(nNPlMds),stat=allocStat)
        call testAllocErr(allocStat,70)
        nPl(1)=0
        maxNPl=0
      end if

    else                             ! NORMAL MODE

      ! currently, normal mode implies:
      !  – only AMHipN and/or RV data present


      ! #-planets modes

      if (isUndef(maxNPl)) then
        nNPlMds=1
        allocate(nPl(nNPlMds),stat=allocStat)
        call testAllocErr(allocStat,71)
        nPl(1)=1
        maxNPl=1
      end if

    end if                           ! (mdlProps%bin)


    call setParsDQs(mdlProps%bin,dtProps,maxNPl,periodogram=usrPeriodogram)
    mdlProps%RVAccel=isDef(pRole%aV)


    ! transit times
    
    if (isDef(transTmStart)) then
      if (mdlProps%bin .OR. (.NOT. mdlProps%bin .AND. all(isUndef(qRoleByPl(1:maxNPl)%tT1)))) &
       call err("calculating transit times not implemented for given mode and data")
    else if (any(isDef(qRoleByPl(1:maxNPl)%tT1))) then
      transTmStart=dtProps%tRge%uBnd
    end if



    ! some checks which require nNPlMds being set

    if (.NOT. (onlySpdMeas .OR. usrReadSamp .OR. isDef(usrMaxLen) .OR. isDef(usrMaxRuntmS) .OR. &
     all(isDef(usrMaxRuntmInMdS(1:nNPlMds)) .OR. isDef(usrMaxLenInMd(1:nNPlMds))))) &
     call err("max. duration of analysis or max. length neither set generally nor for every mode")

    if (any(isUndef(nThin(1:nNPlMds)) .AND. .NOT. ((isDef(usrMaxLen) .OR. isDef(usrMaxLenInMd(1:nNPlMds))) .AND. &
     (isDef(usrMaxRuntmS) .OR. isDef(usrMaxRuntmInMdS(1:nNPlMds)))))) &
     call err("for auto-thinning, max. duration of analysis and max. length must both be given")

    if (any(.NOT. usrAdjustTPar(1:nNPlMds) .AND. isDef(usrTParAdjDurS(1:nNPlMds)))) &
     call err("setting maximum duration of tempering-parameters adjustment when tempering parameters are not adjusted is invalid")


    ! # local pars. & DQs

    do i=1,nNPlMds
      nPars(i)=nParsSys+merge(nParsSysC,0,nPl(i) > 0 .OR. mdlProps%bin)+nPl(i)*nGParsPl
      nDQs(i)=nDQsSys+merge(nDQsSysC,0,nPl(i) > 0 .OR. mdlProps%bin)+nPl(i)*nGDQsPl
    end do

    nParsMax=nPars(nNPlMds)


    if (usrPeriodogram .AND. .NOT. (mdlProps%bin .OR. all(nPl(:nNPlMds) == 1))) &
     call err("periodogram can only be calculated in 1-planet or binary mode")


    ! some local par. properties

    do p=1,nParsMax

      j=gIPars(p)
      parIsCyc(p)=G_PAR_IS_CYC(j)
      parPeriod(p)=G_PAR_PERIOD(j)

      ! for makeChain
      if (parIsCyc(p)) then
        parInvPeriod(p)=1._dp/parPeriod(p)
      else
        parInvPeriod(p)=0._dp       ! dummy
      end if

      priTypes(p)=G_PRI_TYPES(j)
      parHasEpoch(p)=G_PAR_HAS_EPOCH(j)

    end do


    ! for f priors

    where (isDef(nPeriodsRge(1:nNPlMds)))
      priRgeFByNPeriods(1:nNPlMds)%lBnd=nPeriodsRge(1:nNPlMds)%lBnd/dtProps%sngTimeSpansRge%lBnd
      priRgeFByNPeriods(1:nNPlMds)%uBnd=nPeriodsRge(1:nNPlMds)%uBnd/dtProps%sngTimeSpansRge%uBnd
    end where
    if (any(isDef(nPeriodsRge(1:nNPlMds)) .AND. [(priRgeFByNPeriods(i)%width() <= 0, i=1,nNPlMds)])) &
     call err(trim(CL_OPTS(I_CL_OPTS%nPeriodsRange)%short)//" option cannot be used due to the specific time spans of the data")



    if (usrSavSamp) savSamp%filNmSav=trim(outDir)//trim(runIdent)//"_"//trim(fmtFilNm(FIL_NM_PART_SAV_SAMP))// &
     FIL_EXT_SAV_SAMP

    call setMdlFctPtrs(dtProps,mdlProps)


  end subroutine prepAnalysis




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine prepNPlMd
!
!   prepares pars. for a #-planets mode
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j, &
     p, &
     q, &
     p2, &
     p3, &
     iSet
    real(dp) :: &
     sumKMax
    logical :: &
     parsErr, &
     altSuffxPlt
    character :: &
     cBrkt1, &
     cBrkt2
    character(2) :: &
     suffx, &
     suffxPlt
    character(STR_LEN_STD) :: &
     cTmp


    ! # local pars. & DQs

    parsErr=.FALSE.
    if (usrReadSamp .AND. curNPars /= nPars(iNPlMd)) parsErr=.TRUE. ! read-in samples consistency check (part 1/2)
    curNPars=nPars(iNPlMd)
    curNParsFree=curNPars
    curNDQs=merge(nDQs(iNPlMd),0,usrDQs(iNPlMd))
    if (.NOT. usrReadSamp) priLogScl(:curNPars)=G_PAR_LOG_SCL(gIPars(:curNPars)) .AND. &
     log10(intvlBoundsRatio(priRge(:curNPars))) >= LOG_BOUNDS_RATIO_LOG_SCL_MIN
    maPoLogScl(:curNPars)=priLogScl(:curNPars)


    ! set local par. names and read-in samples consistency check (part 2/2)

    do p=1,curNPars
      j=gIPars(p)
      altSuffxPlt=scan(trim(G_PAR_NMS(j)),QTY_NM_SUFFX_DELIM_STD) > 0 .AND. pltPSTricks
      if (isDef(curNPl) .AND. curNPl > 1 .AND. isDef(iPlByPar(p))) then
        ! planet-related par.: suffix for pl.
        suffx=QTY_NM_SUFFX_DELIM_STD//trim(niceFmt(iPlByPar(p)))
        suffxPlt=merge(QTY_NM_SUFFX_DELIM_ALT//trim(niceFmt(iPlByPar(p))),suffx,altSuffxPlt)
      else if (any(isDef(iDtSetByPar(:curNPars)) .AND. iDtSetByPar(:curNPars) > 1) .AND. isDef(iDtSetByPar(p))) then
        ! system-related par., once per data set: suffix for data set
        suffx=QTY_NM_SUFFX_DELIM_STD//trim(niceFmt(iDtSetByPar(p)))
        suffxPlt=merge(QTY_NM_SUFFX_DELIM_ALT//trim(niceFmt(iDtSetByPar(p))),suffx,altSuffxPlt)
      else
        ! system-related par., once for the system: no suffix
        suffx=""
        suffxPlt=""
      end if
      cTmp=trim(G_PAR_NMS(j))//trim(suffx)
      if (usrReadSamp .AND. trim(cTmp) /= trim(parNms(p))) then
        parsErr=.TRUE.
        exit
      else
        parNms(p)=cTmp
        if (pltPSTricks) then
          parNmsPlt(p)="${"//trim(G_PAR_NMS_TEX(j))//"}"//trim(suffxPlt)//"$"
        else
          parNmsPlt(p)=trim(G_PAR_NMS(j))//trim(suffxPlt)
        end if
      end if
    end do

    if (parsErr) call err(trim("saved-samples file contains invalid set of parameters"// &
     mrg("; please try again supplying the original data","",dtProps%nSets == 0)))


    ! set local DQ names

    do q=1,curNDQs
      j=gIDQs(q)
      altSuffxPlt=scan(trim(G_DQ_NMS(j)),QTY_NM_SUFFX_DELIM_STD) > 0 .AND. pltPSTricks
      if (scan(trim(G_DQ_NMS(j))," ") > 0) then
        cBrkt1="("
        cBrkt2=")"
      else
        cBrkt1=""
        cBrkt2=""
      end if
      if (isDef(curNPl) .AND. curNPl > 1 .AND. isDef(iPlByDQ(p))) then
        ! planet-related DQ: suffix for pl.
        suffx=QTY_NM_SUFFX_DELIM_STD//trim(niceFmt(iPlByDQ(p)))
        suffxPlt=merge(QTY_NM_SUFFX_DELIM_ALT//trim(niceFmt(iPlByDQ(p))),suffx,altSuffxPlt)
      else if (any(isDef(iDtSetByDQ(:curNDQs)) .AND. iDtSetByDQ(:curNDQs) > 1) .AND. isDef(iDtSetByDQ(p))) then
        ! system-related DQ, once per data set: suffix for data set
        suffx=QTY_NM_SUFFX_DELIM_STD//trim(niceFmt(iDtSetByDQ(p)))
        suffxPlt=merge(QTY_NM_SUFFX_DELIM_ALT//trim(niceFmt(iDtSetByDQ(p))),suffx,altSuffxPlt)
      else
        ! system-related DQ, once for the system: no suffix
        suffx=""
        suffxPlt=""
        cBrkt1=""
        cBrkt2=""
      end if
      DQNms(q)=trim(cBrkt1)//trim(G_DQ_NMS(j))//trim(cBrkt2)//trim(suffx)
      if (pltPSTricks) then
        DQNmsPlt(q)="${"//trim(cBrkt1)//trim(G_DQ_NMS_TEX(j))//trim(cBrkt2)//"}"//trim(suffxPlt)//"$"
      else
        DQNmsPlt(q)=trim(cBrkt1)//trim(G_DQ_NMS(j))//trim(cBrkt2)//trim(suffxPlt)
      end if
    end do


    if (usrReadSamp) then

      call setEBounds

    else

      call undef(priLBnd)
      call undef(priUBnd)
      call undef(priKnees)
      call undef(priRge)
      parFixed=.FALSE.


      ! prior for f (or f_i) if --n-periods-range option has been used

      if (isDef(priRgeFByNPeriods(iNPlMd))) then
        do p=1,curNPars
          if (gIPars(p) == G_I_PAR_ROLE%f) then
            priLBnd(p)=priRgeFByNPeriods(iNPlMd)%lBnd
            priUBnd(p)=priRgeFByNPeriods(iNPlMd)%uBnd
          end if
        end do
      end if


      ! prior ranges set by user

      do i=1,tmpNPriRges(iNPlMd)
        j=posInArr(parNms(:curNPars),tmpPriRgeNms(iNPlMd,i))
        if (isUndef(j)) then
          call warn("parameter '"//trim(tmpPriRgeNms(iNPlMd,i))//"' not recognised in current mode")
        else
          if (gIPars(j) == G_I_PAR_ROLE%f .AND. isDef(nPeriodsRge(iNPlMd))) &
           call err("prior range of "//trim(parNms(j))//" cannot be set when "//trim(CL_OPTS(I_CL_OPTS%nPeriodsRange)%short)// &
           " option is used")
          if (.NOT. intvlIsInIntvl(tmpPriRge(iNPlMd,i),G_STRICT_PRI_RGE(gIPars(j)))) &
           call err("prior range for "//trim(parNms(j))//" must be a subset of ["//trim(niceFmt(G_STRICT_PRI_RGE(gIPars(j))%lBnd, &
           OPT_REMOVE_0))//","//trim(niceFmt(G_STRICT_PRI_RGE(gIPars(j))%uBnd,OPT_REMOVE_0))//"]")
          if (isDef(tmpPriTy(iNPlMd,i))) priTypes(j)=tmpPriTy(iNPlMd,i)
          if (isDef(tmpPriVce(iNPlMd,i))) priVce(j)=tmpPriVce(iNPlMd,i)
          call setPri(j,tmpPriRge(iNPlMd,i))
          if (priW(j) == 0) parFixed(j)=.TRUE.
        end if
      end do


      ! priors from files

      do i=1,tmpNPriFromFil(iNPlMd)
        j=posInArr(parNms(:curNPars),tmpPriNmsFromFil(iNPlMd,i))
        if (isUndef(j)) then
          call warn("parameter '"//trim(tmpPriNmsFromFil(iNPlMd,i))//"' not recognised in current mode")
        else
          if (gIPars(j) == G_I_PAR_ROLE%f .AND. isDef(nPeriodsRge(iNPlMd))) &
           call err("prior range of "//trim(parNms(j))//" cannot be set when "//trim(CL_OPTS(I_CL_OPTS%nPeriodsRange)%short)// &
           " option is used")
          call setPriFromFil(j,tmpPriFilNms(iNPlMd,i))
          priTypes(j)=PRI_TY%fil
        end if
      end do

      
      ! prior bounds on e (for precalc. of ecc. anom.)
      
      if (psdNPl > 0) call setEBounds


      if (mdlProps%bin) then         ! BINARY MODE


        ! special prior ranges

        if (dtProps%onlyDtTy == DT_TY%AMAngPos) then
          p=pRole%OmegaNode
          call setPri(p,tyIntvlDbl(0,PI),onlyIfUndef=.TRUE.) ! Omega refers to 1st node (0<=Omega<pi) in this case, cf. [09.09.2010/1]
        else
          p2=pRole%e
          call setIfUndef(priUBnd(pRole%K1),.5_dp*sqrt(1._dp-priLBnd(p2)**2)*dtPropsRV%obsRge2SgmCompo(1)%width())
          call setIfUndef(priUBnd(pRole%K2),.5_dp*sqrt(1._dp-priLBnd(p2)**2)*dtPropsRV%obsRge2SgmCompo(2)%width())
        end if


      else if (dtPropsRV%nRec > 0) then

        sumKMax=0
        do i=1,curNPl                ! i: planet index

          ! prior ubound for K_i

          p=pRoleByPl(i)%K           ! K_i
          p3=pRoleByPl(i)%e      ! e_i

          if (usrKMaxFromVel(iNPlMd)) then
            call setIfUndef(priUBnd(p),.5_dp*sqrt(1._dp-priLBnd(p3)**2)*dtPropsRV%obsDiff2SgmPerSetMax)
          else
            p2=pRoleByPl(i)%f    ! f_i
            call setIfUndef(priUBnd(p2),gStdPriRge(G_I_PAR_ROLE%f)%uBnd)
            call setIfUndef(priUBnd(p), &
             real(CUBE_ROOT_2_PI_GRAVC*(priUBnd(p2)/D_IN_S)**(1._dp/3)*mdlProps%mSinIMax(iNPlMd)/mdlProps%mS**(2._dp/3),dp))
          end if
          sumKMax=sumKMax+priUBnd(p)

        end do

      end if                         ! (mdlProps%bin)

      ! special prior ranges

      if (dtPropsAMHipN%nRec > 0 .AND. dtPropsAMAngPos%nRec == 0) then
        call setPri(pRole%tauPlus,tyIntvlDbl(gStdPriRge(G_I_PAR_ROLE%tauPlus)%lBnd,JITT_DEFAULT_UBOUND_STDEV_RATIO* &
         dtPropsAMHipN%obsStdDev),onlyIfUndef=.TRUE.)
      else if (dtPropsAMHipN%nRec == 0 .AND. dtPropsAMAngPos%nRec > 0) then
        call setPri(pRole%tauPlus,tyIntvlDbl(gStdPriRge(G_I_PAR_ROLE%tauPlus)%lBnd,JITT_DEFAULT_UBOUND_STDEV_RATIO* &
         dtPropsAMAngPos%obsStdDev),onlyIfUndef=.TRUE.)
      end if

      if (dtPropsRV%nRec > 0) then

        block
          real(dp) :: &
           aVTmp
          
          avTmp=minval([(dtSetsRV(i)%obsRge2Sgm%width()/(dtSetsRV(i)%tRge%width()/YR_IN_D), i=1,dtProps%nSetsOfTy(DT_TY%RV))])
        
          if (mdlProps%bin) then
            call setPri(pRole%V,dtPropsRV%obsRge2SgmSect,onlyIfUndef=.TRUE.)
          else
            do iSet=1,dtProps%nSetsOfTy(DT_TY%RV)
              call setPri(pRoleBySet(iSet)%V,dtSetsRV(iSet)%obsRge2Sgm,onlyIfUndef=.TRUE.)
            end do
          end if
          if (mdlProps%RVAccel) call setPri(pRole%aV,tyIntvlDbl(-aVTmp,aVTmp),onlyIfUndef=.TRUE.)

        end block

        call setPri(pRole%sgmPlus,tyIntvlDbl(gStdPriRge(G_I_PAR_ROLE%sgmPlus)%lBnd,JITT_DEFAULT_UBOUND_STDEV_RATIO* &
         dtPropsRV%obsStdDev),onlyIfUndef=.TRUE.)
        if (priW(pRole%sgmPlus) < 0) priLBnd(pRole%sgmPlus)=G_STRICT_PRI_RGE(G_I_PAR_ROLE%sgmPlus)%lBnd

      end if


      ! standard prior ranges (if unset); set all priors as intervals and widths

      do p=1,nParsMax
        call setPri(p,gStdPriRge(gIPars(p)),onlyIfUndef=.TRUE.)
      end do

    end if                           ! (.NOT. usrReadSamp)


    ! check that not all pars. are fixed; set cycParUsMaxRge for cyclic pars.

    parInfc=.TRUE.
    do p=1,curNPars
      if (parFixed(p)) then
        call dec(curNParsFree)
        parInfc(p)=.FALSE.
      end if
      cycParUsMaxRge(p)=parIsCyc(p) .AND. priW(p) == parPeriod(p)
    end do

    if (curNParsFree == 0) call err("all parameters are fixed")


    if (usrPeriodogram) then
      if (parFixed(pRole%f)) call err("fixing frequency in periodogram mode is not allowed")
      parInfc=.FALSE.
      parInfc(pRole%f)=.TRUE.
    end if

    curNParsInfc=count(parInfc(:curNPars))


    ! zoom ranges for plots set by user

    if (usrZmRge(iNPlMd)) then
      do p=1,curNPars                ! set undefined ranges to prior ranges
        call setIfUndef(zmRgeAllPars(iNPlMd,p),priRge(p))
      end do
    end if
    call undef(DQZmRge)


    if (psdNPl > 0) call preCalcEA


    contains

    subroutine setEBounds            ! sets eccentricity bounds
      implicit none
      real(dp) :: &
       eMax

      if (mdlProps%bin) then
        call setPri(pRole%e,gStdPriRge(G_I_PAR_ROLE%e),onlyIfUndef=.TRUE.) !ERR
        eMin=priLBnd(pRole%e)
        eMax=priUBnd(pRole%e)
      else
        eMin=1._dp
        eMax=0
        do i=1,curNPl                ! i: planet index
          p=pRoleByPl(i)%e           ! e_i
          call setPri(p,gStdPriRge(G_I_PAR_ROLE%e),onlyIfUndef=.TRUE.)
          eMin=min(eMin,priLBnd(p))
          eMax=max(eMax,priUBnd(p))
        end do
      end if
      eDiff=eMax-eMin
    end subroutine

  end subroutine prepNPlMd




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine updtRuntm(iNPlMd)
!
!   updates max. runtime in cur. #-pl. mode
!
!   Arguments:
!     iNPlMd                         #-pl. mode index
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iNPlMd

    if (isDef(usrMaxRuntmS) .OR. isDef(usrMaxRuntmInMdS(iNPlMd))) then
      curUsrMaxRuntmS=nint(min(merge(1._dp*usrMaxRuntmInMdS(iNPlMd),HUGE_DP,isDef(usrMaxRuntmInMdS(iNPlMd))), &
       merge(usrMaxRuntmS-(omp_get_wtime()-tmStartPrg),HUGE_DP,isDef(usrMaxRuntmS))))
    else
      call undef(curUsrMaxRuntmS)
    end if

    if (isDef(usrMaxLen) .OR. isDef(usrMaxLenInMd(iNPlMd))) then
      curUsrMaxLen=min(merge(usrMaxLenInMd(iNPlMd),HUGE_INT,isDef(usrMaxLenInMd(iNPlMd))), &
       merge(usrMaxLen,HUGE_INT,isDef(usrMaxLen)))
    else
      call undef(curUsrMaxLen)
    end if

  end subroutine updtRuntm




end module
