module basicTypes                               ! Basic data types and conversions

! Routines:
! 
! int1ToBool             Returns: boolean corresponding to given integer
! boolToInt1             Returns: integer corresponding to given boolean

  use iso_fortran_env
  implicit none

!–––––––––––––––––––––––––––––––––––––––––––––––––[ DECLARATIONS AND DEFINITIONS ]––––––––––––––––––––––––––––––––––––––––––––––––––


  !========================================================[ CONSTANTS ]============================================================

  integer,parameter :: &
   i1b=INT8, &
   i2b=INT16, &
   i4b=INT32, &
   i8b=INT64, &
   sp=REAL32, &
   dp=REAL64, &
   qp=REAL128, &
   N_ZEROS_FLOAT_MAX=3, &
   HUGE_INT=huge(1), &
   HUGE_INT1=huge(1_i1b)

  real,parameter :: &
   DIFF_EPS_SP=epsilon(1.), &        ! use nearest() intrinsic instead, if possible!
   HUGE_SP=huge(1.)

  real(dp),parameter :: &
   DIFF_EPS_DP=epsilon(1._dp), &     ! use nearest() intrinsic instead, if possible!
   ONE_PLUS_EPS_DP=1._dp+DIFF_EPS_DP, &
   ONE_MINUS_EPS_DP=1._dp-DIFF_EPS_DP, &
   EPS_DP=tiny(1._dp), &
   HUGE_DP=huge(1._dp)
  
  real(qp),parameter :: &
   HUGE_QP=huge(1._qp)


  !=====================================================[ TYPE DEFINITIONS ]========================================================

  type tyVarTypes
    real :: real
    real(dp) :: dbl
    integer :: int
    integer(i2b) :: int2
    integer(i8b) :: int8
    character :: chr
  end type


!–––––––––––––––––––––––––––––––––––––––––––––––––––––––[ SUBPROGRAM PART ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  contains


!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function int1ToBool(var)
!
!   Returns:                         boolean corresponding to given integer
!
!   Arguments:
!     var                            integer
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i1b) :: &
     var
    logical :: &
     int1ToBool
    intent(in) :: &
     var
    
    int1ToBool=var == 1_i1b
    
  end function int1ToBool




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function boolToInt1(var)
!
!   Returns:                         integer corresponding to given boolean
!
!   Arguments:
!     var                            boolean

!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i1b) :: &
     boolToInt1
    logical :: &
     var
    intent(in) :: &
     var
    
    boolToInt1=merge(1_i1b,0_i1b,var)
    
  end function boolToInt1




end module
