module etc                                              ! Further utilities

! Routines:
! 
! setFlg0                Returns: set of flags (integer) with only one flag being set
! setFlgArr0             Returns: set of flags (integer) with several flags being set
! addFlg                 Returns: set of flags (integer) with one additional flag being set
! addFlgArr              Returns: set of flags (integer) with several additional flags being set
! getFlg                 Returns: whether a flag is set

  use basicTypes
  implicit none

!–––––––––––––––––––––––––––––––––––––––––––––––––[ DECLARATIONS AND DEFINITIONS ]––––––––––––––––––––––––––––––––––––––––––––––––––


  !=====================================================[ TYPE DEFINITIONS ]========================================================

  type tyFlags                       ! FLAGS REPLACING BOOLEAN FUNCTION ARGUMENTS BY ONE INTEGER
    integer :: &
     onlyDt, &
     resid, &
     residVect, &
     fold, &
     chiSq, &
     ignJitt, &
     sorted, &
     xtndRgeX, &
     xtndRgeY, &
     logScl, &
     logSclX, &
     logSclY, &
     adapt, &
     use0, &
     rm0, &
     adjR, &
     noAdvc, &
     simpPrfx, &
     spcPrfx, &
     rvX, &
     rvY, &
     splot, &
     noSty, &
     mkTex, &
     eqAxR, &
     center
  end type


  !========================================================[ CONSTANTS ]============================================================

  ! DEBUGGING

  integer,parameter :: &
   NO_FLAGS=0

  logical,parameter :: &             ! NAMED CONSTANTS FOR FUNCTION ARGUMENTS
   OPT_USE_0=.TRUE., &
   OPT_USE_DOTS=.TRUE., &
   OPT_REMOVE_0=.TRUE., &
   OPT_IS_TIME=.TRUE., &
   OPT_PT_NORMAL=.FALSE., &
   OPT_PT_ADJ_TPAR=.TRUE., &
   OPT_PT_PRESERVE_INIPARS=.TRUE., &
   OPT_PT_CHG_INIPARS=.TRUE., &
   OPT_SORTED=.TRUE., &
   OPT_SORT_DESCENDING=.TRUE., &
   OPT_NO_ADVANCE=.FALSE., &
   OPT_FOR_PLOT=.TRUE., &
   OPT_FOR_STDOUT=.NOT.OPT_FOR_PLOT, &
   OPT_EXTEND_RGE=.TRUE., &
   OPT_SCATTER=.TRUE., &
   OPT_WINDOW=.TRUE., &
   OPT_WITH_RGE=.TRUE., &
   OPT_LONGER=.TRUE., &
   OPT_OUTPUT=.TRUE., &
   OPT_GENERAL=.TRUE., &
   OPT_FOR_PLT=.TRUE., &
   OPT_INVERT=.TRUE., &
   OPT_NO_DQS=.TRUE., &
   OPT_STDEV=.TRUE., &
   OPT_TIMES_EV=.TRUE., &
   OPT_SAMP_UNC=.TRUE.

  type(tyFlags),parameter :: &
   FLG=tyFlags(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25)   ! positions in integer argument corresponding to the individual flags


  !========================================================[ VARIABLES ]============================================================

  integer :: &
   dbgSwitch


!––––––––––––––––––––––––––––––––––––––––––––––––––––––––––[ INTERFACES ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  interface setFlg
   module procedure addFlg
   module procedure setFlg0
   module procedure addFlgArr
   module procedure setFlgArr0
  end interface


!–––––––––––––––––––––––––––––––––––––––––––––––––––––––[ SUBPROGRAM PART ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  contains


!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function setFlg0(iFlg,cond)
!
!   Returns:                         set of flags (integer) with only one flag being set
!
!   Arguments:
!     iFlg                           flag to be set                 
!     cond                           optional condition for setting the flag
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iFlg, &
     setFlg0
    logical :: &
     cond
    optional :: &
     cond
    intent(in) :: &
     iFlg, &
     cond

    setFlg0=setFlg(NO_FLAGS,iFlg,.NOT. present(cond) .OR. (present(cond) .AND. cond))

  end function setFlg0




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function setFlgArr0(iFlgArr,cond)
!
!   Returns:                         set of flags (integer) with several flags being set
!
!   Arguments:
!     iFlgArr                        array of flags to be set
!     cond                           optional condition for setting the flags
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iFlgArr(:), &
     setFlgArr0, &
     i
    logical :: &
     cond(:)
    optional :: &
     cond
    intent(in) :: &
     iFlgArr, &
     cond

    setFlgArr0=NO_FLAGS
    do i=1,ubound(iFlgArr,1)
      setFlgArr0=setFlg(setFlgArr0,iFlgArr(i),.NOT. present(cond) .OR. (present(cond) .AND. cond(i)))
    end do

  end function setFlgArr0




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function addFlg(flags,iFlg,cond)
!
!   Returns:                         set of flags (integer) with one additional flag being set
!
!   Arguments:
!     flags                          original set of flags
!     iFlg                           flag to be set
!     cond                           optional condition for setting the flag
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iFlg, &
     flags, &
     addFlg
    logical :: &
     cond
    optional :: &
     cond
    intent(in) :: &
     flags, &
     iFlg, &
     cond

    if (.NOT. present(cond) .OR. (present(cond) .AND. cond)) then
      addFlg=ibset(flags,iFlg)
    else
      addFlg=flags
    end if

  end function addFlg




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function addFlgArr(flags,iFlgArr,cond)
!
!   Returns:                         set of flags (integer) with several additional flags being set
!
!   Arguments:
!     flags                          original set of flags
!     iFlgArr                        array of flags to be set
!     cond                           optional condition for setting the flags
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iFlgArr(:), &
     flags, &
     addFlgArr, &
     i
    logical :: &
     cond(:)
    optional :: &
     cond
    intent(in) :: &
     flags, &
     iFlgArr, &
     cond

    addFlgArr=flags
    do i=1,ubound(iFlgArr,1)
      addFlgArr=setFlg(addFlgArr,iFlgArr(i),.NOT. present(cond) .OR. (present(cond) .AND. cond(i)))
    end do

  end function addFlgArr




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function getFlg(flags,iFlg)
!
!   Returns:                         whether a flag is set
!
!   Arguments:
!     flags                          set of flags
!     iFlg                           flag in question
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     flags, &
     iFlg
    logical :: &
     getFlg
    intent(in) :: &
     flags, &
     iFlg

    getFlg=ibits(flags,iFlg,1) == 1

  end function getFlg




end module
