module sorting                                  ! Sorting and other tasks on arrays

! Routines:
! 
! compIntLT              Returns: whether a < b
! compIntLE              Returns: whether a <= b
! compIntGE              Returns: whether a >= b
! compIntGT              Returns: whether a > b
! compRealLT             Returns: whether a < b
! compRealLE             Returns: whether a <= b
! compRealGE             Returns: whether a >= b
! compRealGT             Returns: whether a > b
! compDblLT              Returns: whether a < b
! compDblLE              Returns: whether a <= b
! compDblGE              Returns: whether a >= b
! compDblGT              Returns: whether a > b
! compQPLT               Returns: whether a < b
! compQPLE               Returns: whether a <= b
! compQPGE               Returns: whether a >= b
! compQPGT               Returns: whether a > b
! replaceInt             Returns: given array with a certain element replaced by a certain value
! replaceDbl             Returns: given array with a certain element replaced by a certain value
! replaceReal            Returns: given array with a certain element replaced by a certain value
! swapInt                Returns: swapped arguments
! swapInt8               Returns: swapped arguments
! swapReal               Returns: swapped arguments
! swapDbl                Returns: swapped arguments
! swapQP                 Returns: swapped arguments
! swapStr                Returns: swapped arguments
! swapElemInt            Returns: given array with two elements swapped
! swapElemReal           Returns: given array with two elements swapped
! swapElemDbl            Returns: given array with two elements swapped
! delSubsetInt           Returns: given array with a certain subset deleted
! delSubsetReal          Returns: given array with a certain subset deleted
! delSubsetDbl           Returns: given array with a certain subset deleted
! shft4DP                Shifts the values of the last three arguments into the first three arguments
! posInArrInt4           Returns: index of first element of an array equal to a certain value
! posInArrInt8           Returns: index of first element of an array equal to a certain value
! posInArrReal           Returns: index of first element of an array equal to a certain value
! posInArrDbl            Returns: index of first element of an array equal to a certain value
! posInArrStr            Returns: index of first element of an array equal to a certain value
! posInSortedArrDbl      Returns: first pos. in 1-d array where element is >= val (or > val, if onlyGT)
! findMaxValIndxDbl      Determines the maximum (or minimum) element value and its index in an array
! findMaxValIndxDblInt8  Determines the maximum (or minimum) element value and its index in an array
! duplicateArrDbl        Duplicates elements of the first halves of two or three arrays according to a certain threshold
! foldArrDbl             "Folds" points according to their abscissae, an offset and a period
! heapSortInt            Performs heapsort on an array
! heapSortReal           Performs heapsort on an array
! heapSortDbl            Performs heapsort on an array
! heapSortQP             Performs heapsort on an array
! indxHeapSortReal       Performs heapsort to produce an index array which maps the original array to a sorted variant
! indxHeapSortDbl        Performs heapsort to produce an index array which maps the original array to a sorted variant
! indxHeapSortQP         Performs heapsort to produce an index array which maps the original array to a sorted variant

  use basicTypes
  use strings
  implicit none

!–––––––––––––––––––––––––––––––––––––––––––––––––[ DECLARATIONS AND DEFINITIONS ]––––––––––––––––––––––––––––––––––––––––––––––––––


!––––––––––––––––––––––––––––––––––––––––––––––––––––––––––[ INTERFACES ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––


  ! BASIC PROCEDURES

  interface replace
    module procedure replaceInt
    module procedure replaceReal
    module procedure replaceDbl
  end interface

  interface swap
    module procedure swapInt
    module procedure swapInt8
    module procedure swapReal
    module procedure swapDbl
    module procedure swapQP
    module procedure swapStr
  end interface

  interface swapElem
    module procedure swapElemInt
    module procedure swapElemReal
    module procedure swapElemDbl
  end interface
  
  interface delSubset
    module procedure delSubsetInt
    module procedure delSubsetReal
    module procedure delSubsetDbl
  end interface


  ! FINDING ARRAY ELEMENTS

  interface posInArr
    module procedure posInArrInt4
    module procedure posInArrInt8
    module procedure posInArrReal
    module procedure posInArrDbl
    module procedure posInArrStr
  end interface

  interface posInSortedArr
    module procedure posInSortedArrDbl
  end interface

  interface findMaxValIndx
    module procedure findMaxValIndxDbl
    module procedure findMaxValIndxDblInt8
  end interface


  ! DUPLICATING AND FOLDING

  interface duplicateArr
    module procedure duplicateArrDbl
  end interface

  interface foldArr
    module procedure foldArrDbl
  end interface


  ! SORTING

  interface sort
    module procedure heapSortInt
    module procedure heapSortReal
    module procedure heapSortDbl
    module procedure heapSortQP
  end interface

  interface indxArr
    module procedure indxHeapSortReal
    module procedure indxHeapSortDbl
    module procedure indxHeapSortQP
  end interface



!–––––––––––––––––––––––––––––––––––––––––––––––––––––––[ SUBPROGRAM PART ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  contains


  !====================================================[ BASIC PROCEDURES ]=========================================================


!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compIntLT(a,b)
!
!   Returns:                         whether a < b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     a, &
     b
    logical :: &
     compIntLT
    intent(in) :: &
     a, &
     b

    compIntLT=a < b

  end function compIntLT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compIntLE(a,b)
!
!   Returns:                         whether a <= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     a, &
     b
    logical :: &
     compIntLE
    intent(in) :: &
     a, &
     b

    compIntLE=a <= b

  end function compIntLE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compIntGE(a,b)
!
!   Returns:                         whether a >= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     a, &
     b
    logical :: &
     compIntGE
    intent(in) :: &
     a, &
     b

    compIntGE=a >= b

  end function compIntGE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compIntGT(a,b)
!
!   Returns:                         whether a > b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     a, &
     b
    logical :: &
     compIntGT
    intent(in) :: &
     a, &
     b

    compIntGT=a > b

  end function compIntGT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compRealLT(a,b)
!
!   Returns:                         whether a < b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real :: &
     a, &
     b
    logical :: &
     compRealLT
    intent(in) :: &
     a, &
     b

    compRealLT=a < b

  end function compRealLT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compRealLE(a,b)
!
!   Returns:                         whether a <= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real :: &
     a, &
     b
    logical :: &
     compRealLE
    intent(in) :: &
     a, &
     b

    compRealLE=a <= b

  end function compRealLE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compRealGE(a,b)
!
!   Returns:                         whether a >= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real :: &
     a, &
     b
    logical :: &
     compRealGE
    intent(in) :: &
     a, &
     b

    compRealGE=a >= b

  end function compRealGE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compRealGT(a,b)
!
!   Returns:                         whether a > b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real :: &
     a, &
     b
    logical :: &
     compRealGT
    intent(in) :: &
     a, &
     b

    compRealGT=a > b

  end function compRealGT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compDblLT(a,b)
!
!   Returns:                         whether a < b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b
    logical :: &
     compDblLT
    intent(in) :: &
     a, &
     b

    compDblLT=a < b

  end function compDblLT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compDblLE(a,b)
!
!   Returns:                         whether a <= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b
    logical :: &
     compDblLE
    intent(in) :: &
     a, &
     b

    compDblLE=a <= b

  end function compDblLE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compDblGE(a,b)
!
!   Returns:                         whether a >= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b
    logical :: &
     compDblGE
    intent(in) :: &
     a, &
     b

    compDblGE=a >= b

  end function compDblGE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compDblGT(a,b)
!
!   Returns:                         whether a > b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b
    logical :: &
     compDblGT
    intent(in) :: &
     a, &
     b

    compDblGT=a > b

  end function compDblGT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compQPLT(a,b)
!
!   Returns:                         whether a < b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(qp) :: &
     a, &
     b
    logical :: &
     compQPLT
    intent(in) :: &
     a, &
     b

    compQPLT=a < b

  end function compQPLT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compQPLE(a,b)
!
!   Returns:                         whether a <= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(qp) :: &
     a, &
     b
    logical :: &
     compQPLE
    intent(in) :: &
     a, &
     b

    compQPLE=a <= b

  end function compQPLE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compQPGE(a,b)
!
!   Returns:                         whether a >= b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(qp) :: &
     a, &
     b
    logical :: &
     compQPGE
    intent(in) :: &
     a, &
     b

    compQPGE=a >= b

  end function compQPGE




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function compQPGT(a,b)
!
!   Returns:                         whether a > b
!
!   Arguments:
!     a                              number
!     b                              number
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(qp) :: &
     a, &
     b
    logical :: &
     compQPGT
    intent(in) :: &
     a, &
     b

    compQPGT=a > b

  end function compQPGT




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function replaceInt(arr,i,rep)
!
!   Returns:                         given array with a certain element replaced by a certain value
!
!   Arguments:
!     arr                            original array
!     i                              index of element to be replaced
!     rep                            replacement value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     arr(:), &
     replaceInt(size(arr,1)), &
     rep, &
     i
    intent(in) :: &
     arr, &
     i, &
     rep

    replaceInt=arr
    replaceInt(i)=rep

  end function replaceInt




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function replaceDbl(arr,i,rep)
!
!   Returns:                         given array with a certain element replaced by a certain value
!
!   Arguments:
!     arr                            original array
!     i                              index of element to be replaced
!     rep                            replacement value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i
    real(dp) :: &
     arr(:), &
     replaceDbl(size(arr,1)), &
     rep
    intent(in) :: &
     arr, &
     i, &
     rep

    replaceDbl=arr
    replaceDbl(i)=rep

  end function replaceDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function replaceReal(arr,i,rep)
!
!   Returns:                         given array with a certain element replaced by a certain value
!
!   Arguments:
!     arr                            original array
!     i                              index of element to be replaced
!     rep                            replacement value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i
    real :: &
     arr(:), &
     replaceReal(size(arr,1)), &
     rep
    intent(in) :: &
     arr, &
     i, &
     rep

    replaceReal=arr
    replaceReal(i)=rep

  end function replaceReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapInt(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     a, &
     b, &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapInt




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapInt8(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     a, &
     b, &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapInt8




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapReal(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real :: &
     a, &
     b, &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapDbl(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b, &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapQP(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(qp) :: &
     a, &
     b, &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapQP




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    elemental subroutine swapStr(a,b)
!
!   Returns:                         swapped arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    character(*) :: &
     a, &
     b
    character(len(a)) :: &
     tmp
    intent(inout) :: &
     a, &
     b

    tmp=a
    a=b
    b=tmp

  end subroutine swapStr




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function swapElemInt(arr,i,j)
!
!   Returns:                         given array with two elements swapped
!
!   Arguments:
!     arr                            original array
!     i                              element index
!     j                              element index
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     arr(:), &
     swapElemInt(size(arr,1)), &
     i, &
     j
    intent(in) :: &
     arr, &
     i, &
     j

    swapElemInt=arr
    call swap(swapElemInt(i),swapElemInt(j))

  end function swapElemInt




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function swapElemReal(arr,i,j)
!
!   Returns:                         given array with two elements swapped
!
!   Arguments:
!     arr                            original array
!     i                              element index
!     j                              element index
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j
    real :: &
     arr(:), &
     swapElemReal(size(arr,1))
    intent(in) :: &
     arr, &
     i, &
     j

    swapElemReal=arr
    call swap(swapElemReal(i),swapElemReal(j))

  end function swapElemReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function swapElemDbl(arr,i,j)
!
!   Returns:                         given array with two elements swapped
!
!   Arguments:
!     arr                            original array
!     i                              element index
!     j                              element index
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j
    real(dp) :: &
     arr(:), &
     swapElemDbl(size(arr,1))
    intent(in) :: &
     arr, &
     i, &
     j

    swapElemDbl=arr
    call swap(swapElemDbl(i),swapElemDbl(j))

  end function swapElemDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function delSubsetInt(arr,i,l)
!
!   Returns:                         given array with a certain subset deleted
!
!   Arguments:
!     arr                            original array
!     i                              index of first element to be deleted
!     l                              # elements to be deleted
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     l, &
     N
    integer :: &
     arr(:), &
     delSubsetInt(size(arr,1)-l)
    intent(in) :: &
     arr, &
     i, &
     l
    
    N=size(arr,1)

    delSubsetInt(1:i-1)=arr(1:i-1)
    delSubsetInt(i:N-l)=arr(i+l:N)

  end function delSubsetInt




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function delSubsetReal(arr,i,l)
!
!   Returns:                         given array with a certain subset deleted
!
!   Arguments:
!     arr                            original array
!     i                              index of first element to be deleted
!     l                              # elements to be deleted
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     l, &
     N
    real :: &
     arr(:), &
     delSubsetReal(size(arr,1)-l)
    intent(in) :: &
     arr, &
     i, &
     l
    
    N=size(arr,1)

    delSubsetReal(1:i-1)=arr(1:i-1)
    delSubsetReal(i:N-l)=arr(i+l:N)

  end function delSubsetReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function delSubsetDbl(arr,i,l)
!
!   Returns:                         given array with a certain subset deleted
!
!   Arguments:
!     arr                            original array
!     i                              index of first element to be deleted
!     l                              # elements to be deleted
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     l, &
     N
    real(dp) :: &
     arr(:), &
     delSubsetDbl(size(arr,1)-l)
    intent(in) :: &
     arr, &
     i, &
     l
    
    N=size(arr,1)

    delSubsetDbl(1:i-1)=arr(1:i-1)
    delSubsetDbl(i:N-l)=arr(i+l:N)

  end function delSubsetDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine shft4DP(a,b,c,d)
!
!   shifts the values of the last three arguments into the first three arguments
!
!   Arguments:
!     a                              argument
!     b                              argument
!     c                              argument
!     d                              argument
!
!   from [324] (mnbrak.f90)
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     a, &
     b, &
     c, &
     d
    intent(out) :: &
     a
    intent(inout) :: &
     b, &
     c
    intent(in) :: &
     d

    a=b
    b=c
    c=d

  end subroutine shft4DP




  !=================================================[ FINDING ARRAY ELEMENTS ]======================================================




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInArrInt4(arr,val)
!
!   Returns:                         index of first element of an array equal to a certain value
!
!   Arguments:
!     arr                            array
!     val                            value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     posInArrInt4, &
     arr(:), &
     val, &
     i
    intent(in) :: &
     arr, &
     val


    i=lbound(arr,1)

    do
      if (i > ubound(arr,1)) then
        call undef(posInArrInt4)
        return
      else if (arr(i) == val) then
        posInArrInt4=i
        return
      else
        i=i+1
      end if
    end do

  end function posInArrInt4




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInArrInt8(arr,val)
!
!   Returns:                         index of first element of an array equal to a certain value
!
!   Arguments:
!     arr                            array
!     val                            value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     posInArrInt8, &
     arr(:), &
     val, &
     i
    intent(in) :: &
     arr, &
     val


    i=lbound(arr,1)

    do
      if (i > ubound(arr,1)) then
        call undef(posInArrInt8)
        return
      else if (arr(i) == val) then
        posInArrInt8=i
        return
      else
        i=i+1
      end if
    end do

  end function posInArrInt8




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInArrReal(arr,val)
!
!   Returns:                         index of first element of an array equal to a certain value
!
!   Arguments:
!     arr                            array
!     val                            value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     posInArrReal, &
     i
    real :: &
     arr(:), &
     val
    intent(in) :: &
     arr, &
     val


    i=lbound(arr,1)

    do
      if (i > ubound(arr,1)) then
        call undef(posInArrReal)
        return
      else if (arr(i) == val) then
        posInArrReal=i
        return
      else
        i=i+1
      end if
    end do

  end function posInArrReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInArrDbl(arr,val)
!
!   Returns:                         index of first element of an array equal to a certain value
!
!   Arguments:
!     arr                            array
!     val                            value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     posInArrDbl, &
     i
    real(dp) :: &
     arr(:), &
     val
    intent(in) :: &
     arr, &
     val


    i=lbound(arr,1)

    do
      if (i > ubound(arr,1)) then
        call undef(posInArrDbl)
        return
      else if (arr(i) == val) then
        posInArrDbl=i
        return
      else
        i=i+1
      end if
    end do

  end function posInArrDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInArrStr(arr,val)
!
!   Returns:                         index of first element of an array equal to a certain value
!
!   Arguments:
!     arr                            array
!     val                            value
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     posInArrStr, &
     i
    character(*) :: &
     arr(:), &
     val
    intent(in) :: &
     arr, &
     val


    i=lbound(arr,1)

    do
      if (i > ubound(arr,1)) then
        call undef(posInArrStr)
        return
      else if (trim(arr(i)) == trim(val)) then
        posInArrStr=i
        return
      else
        i=i+1
      end if
    end do

  end function posInArrStr




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function posInSortedArrDbl(arr,val,onlyGT)
!
!   Returns:                         first pos. in 1-d array where element is >= val (or > val, if onlyGT)
!
!   Arguments:
!     arr                            array sorted in ascending order
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     posInSortedArrDbl, &
     l, &
     r, &
     m
    real(dp) :: &
     arr(:), &
     val, &
     p
    logical :: &
     onlyGT
    procedure(compDblGE) :: &
     comp
    pointer :: &
     comp
    intent(in) :: &
     arr, &
     val, &
     onlyGT

    if (onlyGT) then
      comp=>compDblGT
    else
      comp=>compDblGE
    end if

    l=1
    r=size(arr,1)

    if (comp(arr(l),val)) then
      posInSortedArrDbl=l
    else if (.NOT. comp(arr(r),val)) then
      call undef(posInSortedArrDbl)
    else
      do
        m=ceiling(.5_dp*(l+r))
        p=arr(m)
        if (comp(p,val)) then
          r=m
        else
          l=m
        end if
        if (r-l == 1) then
          posInSortedArrDbl=r
          exit
        end if
      end do
    end if

  end function posInSortedArrDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine findMaxValIndxDbl(arr,valMax,iMax,inv)
!
!   determines the maximum (or minimum) element value and its index in an array
!
!   Arguments:
!     arr                            array in question
!     valMax                         max. value
!     iMax                           index of max. value
!     inv                            whether the minimum should be sought
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     iMax, &
     i
    real(dp) :: &
     arr(:), &
     valMax
    logical :: &
     inv, &
     found
    procedure(compDblGT) :: &
     comp
    pointer :: &
     comp
    optional :: &
     inv
    intent(in) :: &
     arr, &
     inv
    intent(out) :: &
     valMax, &
     iMax

    if (present(inv) .AND. inv) then
      comp=>compDblLT
      valMax=huge(valMax)
    else
      comp=>compDblGT
      valMax=-huge(valMax)
    end if
    found=.FALSE.

    do i=1,ubound(arr,1)
      if (comp(arr(i),valMax)) then
        valMax=arr(i)
        iMax=i
        found=.TRUE.
      end if
    end do

    if (.NOT. found) then
      call undef(valMax)
      call undef(iMax)
    end if

  end subroutine findMaxValIndxDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine findMaxValIndxDblInt8(arr,valMax,iMax,inv)
!
!   determines the maximum (or minimum) element value and its index in an array
!
!   Arguments:
!     arr                            array in question
!     valMax                         max. value
!     iMax                           index of max. value
!     inv                            whether the minimum should be sought
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     iMax, &
     i
    real(dp) :: &
     arr(:), &
     valMax
    logical :: &
     inv, &
     found
    procedure(compDblGT) :: &
     comp
    pointer :: &
     comp
    optional :: &
     inv
    intent(in) :: &
     arr, &
     inv
    intent(out) :: &
     valMax, &
     iMax

    if (present(inv) .AND. inv) then
      comp=>compDblLT
      valMax=huge(valMax)
    else
      comp=>compDblGT
      valMax=-huge(valMax)
    end if
    found=.FALSE.

    do i=1,ubound(arr,1)
      if (comp(arr(i),valMax)) then
        valMax=arr(i)
        iMax=i
        found=.TRUE.
      end if
    end do

    if (.NOT. found) then
      call undef(valMax)
      call undef(iMax)
    end if

  end subroutine findMaxValIndxDblInt8




  !================================================[ DUPLICATING AND FOLDING ]======================================================




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine duplicateArrDbl(arrX,arrY,thresh,dupX,arrZ)
!
!   duplicates elements of the first halves of two or three arrays according to a certain threshold
!
!   Arguments:
!     arrX                           array of abscissae
!     arrY                           array of ordinates
!     thresh                         threshold
!     dupX                           whether abscissae should also be duplicated
!     arrZ                           additional array of ordinates
!
!   elements i with arrX(i) >= thresh are duplicated to the left, the others to the right
!
!   Requirements:
!     - arrX must be sorted
!     - arrX(i) in [0,2·thresh]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i, &
     j, &
     N, &
     M
    real(dp) :: &
     arrX(:), &
     arrY(:), &
     thresh, &
     arrXOri(size(arrX,1)/2), &
     arrYOri(size(arrY,1)/2), &
     arrZOri(size(arrY,1)/2), &
     arrZ(:)
    logical :: &
     dupX
    optional :: &
     arrZ
    intent(in) :: &
     thresh, &
     dupX
    intent(inout) :: &
     arrX, &
     arrY, &
     arrZ

    N=size(arrX,1)/2
    if (dupX) arrXOri=arrX(:N)
    arrYOri=arrY(:N)
    if (present(arrZ)) arrZOri=arrZ(:N)
    do i=1,N
      if (arrX(i) >= thresh) then
        j=i
        exit
      end if
    end do

    M=N-j+1                          ! # elements >= threshold

    ! elements duplicated to the left
    if (dupX) arrX(1:M)=arrXOri(j:N)-2._dp*thresh
    arrY(1:M)=arrYOri(j:N)
    if (present(arrZ)) arrZ(1:M)=arrZOri(j:N)

    ! original elements
    if (dupX) arrX(M+1:M+N)=arrXOri(1:N)
    arrY(M+1:M+N)=arrYOri(1:N)
    if (present(arrZ)) arrZ(M+1:M+N)=arrZOri(1:N)

    ! elements duplicated to the right
    if (dupX) arrX(M+N+1:M+N+j-1)=arrXOri(1:j-1)+2._dp*thresh
    arrY(M+N+1:M+N+j-1)=arrYOri(1:j-1)
    if (present(arrZ)) arrZ(M+N+1:M+N+j-1)=arrZOri(1:j-1)

  end subroutine duplicateArrDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine foldArrDbl(arrX,arrY,ofs,period,foldX,arrZ)
!
!   "folds" points according to their abscissae, an offset and a period
!
!   Arguments:
!     arrX                           array of abscissae
!     arrY                           array of ordinates
!     ofs                            folding offset
!     foldX                          whether abscissae should also be folded
!     arrZ                           additional array of ordinates
!   
!   returns points sorted by ascending folded x
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    real(dp) :: &
     arrX(:), &
     arrY(:), &
     ofs, &
     period, &
     arrXFold(size(arrX,1)), &
     arrZ(:)
    integer(i8b) :: &
     iArr(size(arrX,1))
    logical :: &
     foldX
    optional :: &
     arrZ
    intent(inout) :: &
     arrX, &
     arrY, &
     arrZ
    intent(in) :: &
     ofs, &
     period, &
     foldX

    arrXFold=modulo((arrX-ofs)/period,1._dp)
    call indxArr(arrXFold,iArr)
    arrY=arrY(iArr)                  ! sort by ascending X
    if (present(arrZ)) arrZ=arrZ(iArr)
    if (foldX) arrX=arrXFold(iArr)

  end subroutine foldArrDbl




  !========================================================[ SORTING ]==============================================================




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine heapSortInt(arr,descend)
!
!   performs heapsort on an array
!
!   Arguments:
!     arr                            array to be sorted
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n
    integer :: &
     arr(:)
    logical :: &
     descend
    procedure(compIntLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(inout) :: &
     arr
    intent(in) :: &
     descend

    if (present(descend) .AND. descend) then
      cmp1=>compIntGT
      cmp2=>compIntLE
    else
      cmp1=>compIntLT
      cmp2=>compIntGE
    end if

    n=size(arr)
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(arr(1),arr(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold
      integer :: &
       a
      intent(in) :: &
       l, &
       r
      a=arr(l)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(j),arr(j+1))) j=j+1
        if (cmp2(a,arr(j))) exit
        arr(jold)=arr(j)
        jold=j
        j=j+j
      end do
      arr(jold)=a
    end subroutine

  end subroutine heapSortInt




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine heapSortReal(arr,descend)
!
!   performs heapsort on an array
!
!   Arguments:
!     arr                            array to be sorted
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n
    real :: &
     arr(:)
    logical :: &
     descend
    procedure(compRealLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(inout) :: &
     arr
    intent(in) :: &
     descend

    if (present(descend) .AND. descend) then
      cmp1=>compRealGT
      cmp2=>compRealLE
    else
      cmp1=>compRealLT
      cmp2=>compRealGE
    end if

    n=size(arr)
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(arr(1),arr(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold
      real :: &
       a
      intent(in) :: &
       l, &
       r
      a=arr(l)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(j),arr(j+1))) j=j+1
        if (cmp2(a,arr(j))) exit
        arr(jold)=arr(j)
        jold=j
        j=j+j
      end do
      arr(jold)=a
    end subroutine

  end subroutine heapSortReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine heapSortDbl(arr,descend)
!
!   performs heapsort on an array
!
!   Arguments:
!     arr                            array to be sorted
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n
    real(dp) :: &
     arr(:)
    logical :: &
     descend
    procedure(compDblLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(inout) :: &
     arr
    intent(in) :: &
     descend

    if (present(descend) .AND. descend) then
      cmp1=>compDblGT
      cmp2=>compDblLE
    else
      cmp1=>compDblLT
      cmp2=>compDblGE
    end if

    n=size(arr)
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(arr(1),arr(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold
      real(dp) :: &
       a
      intent(in) :: &
       l, &
       r
      a=arr(l)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(j),arr(j+1))) j=j+1
        if (cmp2(a,arr(j))) exit
        arr(jold)=arr(j)
        jold=j
        j=j+j
      end do
      arr(jold)=a
    end subroutine

  end subroutine heapSortDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine heapSortQP(arr,descend)
!
!   performs heapsort on an array
!
!   Arguments:
!     arr                            array to be sorted
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n
    real(qp) :: &
     arr(:)
    logical :: &
     descend
    procedure(compQPLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(inout) :: &
     arr
    intent(in) :: &
     descend

    if (present(descend) .AND. descend) then
      cmp1=>compQPGT
      cmp2=>compQPLE
    else
      cmp1=>compQPLT
      cmp2=>compQPGE
    end if

    n=size(arr)
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(arr(1),arr(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold
      real(qp) :: &
       a
      intent(in) :: &
       l, &
       r
      a=arr(l)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(j),arr(j+1))) j=j+1
        if (cmp2(a,arr(j))) exit
        arr(jold)=arr(j)
        jold=j
        j=j+j
      end do
      arr(jold)=a
    end subroutine

  end subroutine heapSortQP




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine indxHeapSortReal(arr,indx,descend)
!
!   performs heapsort to produce an index array which maps the original array to a sorted variant
!
!   Arguments:
!     arr                            array to be sorted
!     indx                           index array
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n, &
     indx(:)
    real :: &
     arr(:)
    logical :: &
     descend
    procedure(compRealLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(in) :: &
     arr, &
     descend
    intent(inout) :: &
     indx

    if (present(descend) .AND. descend) then
      cmp1=>compRealGT
      cmp2=>compRealLE
    else
      cmp1=>compRealLT
      cmp2=>compRealGE
    end if

    n=size(arr)
    do i=1,n
      indx(i)=i
    end do
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(indx(1),indx(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold, &
       ia
      real :: &
       a
      intent(in) :: &
       l, &
       r
      ia=indx(l)
      a=arr(ia)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(indx(j)),arr(indx(j+1)))) j=j+1
        if (cmp2(a,arr(indx(j)))) exit
        indx(jold)=indx(j)
        jold=j
        j=j+j
      end do
      indx(jold)=ia
    end subroutine

  end subroutine indxHeapSortReal




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine indxHeapSortDbl(arr,indx,descend)
!
!   performs heapsort to produce an index array which maps the original array to a sorted variant
!
!   Arguments:
!     arr                            array to be sorted
!     indx                           index array
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n, &
     indx(:)
    real(dp) :: &
     arr(:)
    logical :: &
     descend
    procedure(compDblLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(in) :: &
     arr, &
     descend
    intent(inout) :: &
     indx

    if (present(descend) .AND. descend) then
      cmp1=>compDblGT
      cmp2=>compDblLE
    else
      cmp1=>compDblLT
      cmp2=>compDblGE
    end if

    n=size(arr)
    do i=1,n
      indx(i)=i
    end do
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(indx(1),indx(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold, &
       ia
      real(dp) :: &
       a
      intent(in) :: &
       l, &
       r
      ia=indx(l)
      a=arr(ia)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(indx(j)),arr(indx(j+1)))) j=j+1
        if (cmp2(a,arr(indx(j)))) exit
        indx(jold)=indx(j)
        jold=j
        j=j+j
      end do
      indx(jold)=ia
    end subroutine

  end subroutine indxHeapSortDbl




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine indxHeapSortQP(arr,indx,descend)
!
!   performs heapsort to produce an index array which maps the original array to a sorted variant
!
!   Arguments:
!     arr                            array to be sorted
!     indx                           index array
!     descend                        whether sorting should be in descending order
!
!   adapted from [324]
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer(i8b) :: &
     i, &
     n, &
     indx(:)
    real(qp) :: &
     arr(:)
    logical :: &
     descend
    procedure(compQPLT),pointer :: &
     cmp1, &
     cmp2
    optional :: &
     descend
    intent(in) :: &
     arr, &
     descend
    intent(inout) :: &
     indx

    if (present(descend) .AND. descend) then
      cmp1=>compQPGT
      cmp2=>compQPLE
    else
      cmp1=>compQPLT
      cmp2=>compQPGE
    end if

    n=size(arr)
    do i=1,n
      indx(i)=i
    end do
    do i=n/2,1,-1
      call sift_down(i,n)
    end do
    do i=n,2,-1
      call swap(indx(1),indx(i))
      call sift_down(1_i8b,i-1)
    end do


    contains

    subroutine sift_down(l,r)        ! sifts down an element to its proper level
      implicit none
      integer(i8b) :: &
       l, &
       r, &
       j, &
       jold, &
       ia
      real(qp) :: &
       a
      intent(in) :: &
       l, &
       r
      ia=indx(l)
      a=arr(ia)
      jold=l
      j=l+l
      do
        if (j > r) exit
        if (j < r .AND. cmp1(arr(indx(j)),arr(indx(j+1)))) j=j+1
        if (cmp2(a,arr(indx(j)))) exit
        indx(jold)=indx(j)
        jold=j
        j=j+j
      end do
      indx(jold)=ia
    end subroutine

  end subroutine indxHeapSortQP




end module
