module userOpts                                           ! User options

! Routines:
! 
! getOptIndx             Returns: index of a user option in CL_OPTS array
! tryOptCmd              Returns: whether a string corresponds to a certain command-type user option
! tryOpt1IntAssign       Returns: whether a string corresponds to a certain assignment-type user option
! tryOpt1RealAssign      Returns: whether a string corresponds to a certain assignment-type user option
! tryOpt1DblAssign       Returns: whether a string corresponds to a certain assignment-type user option
! tryOptIntAssign        Returns: whether a string corresponds to a certain assignment-type user option
! tryOptRealAssign       Returns: whether a string corresponds to a certain assignment-type user option
! tryOptDblAssign        Returns: whether a string corresponds to a certain assignment-type user option
! tryOptStrAssign        Returns: whether a string corresponds to a certain assignment-type user option
! printHelp              Prints a help screen including all user options
! printOption            Prints a user option and its description

  use meta
  use basicTypes
  use io
  implicit none

!–––––––––––––––––––––––––––––––––––––––––––––––––[ DECLARATIONS AND DEFINITIONS ]––––––––––––––––––––––––––––––––––––––––––––––––––


  !=====================================================[ TYPE DEFINITIONS ]========================================================

  ! to add a new command-line option:
  !   1. increase N_CL_OPTS
  !   2. extend I_CL_OPTS by appending another number
  !   3. extend tyCLOptInt
  !   4. extend CL_OPTS in the same order

  type tyCLOptInt
    integer :: calcHPDI,calcProb,calcProbAllPars,calcTransitAfter,comment,dataDir,discardFrac,discardPSRScale,epoch, &
     generateData,group,help,helpParsDQs,inferenceLenFrac,inferenceLen,KMaxFromVel,keepSeed,M,mailWhenFinished,maxDur, &
     maxLen,mSinIMax,nPeriodsRange,nPl,nPT,noDQs,noFiles,noGnuplotExec,noInfc,noJointMaPo,noMaPo,noPlt,noPS4PDFExec,noTempParsAdj, &
     onlyPlotData,onlySpeedMeas, &
     outDir,outDirCSV,periodogram,predict,prior,PSRConv,PSTricksPlots,PTNChains,quickPlots,quiet,range,readSamp, &
     savCandCSV,savSamp, &
     scatterPlots,scope,singlePlots,speedMeasDur,stdOutLineLen,swap,tempPars,tempParsAdjDur,thin,value,zmRgeAllPars
  end type
  type tyCLOpt
    character(STR_LEN_STD) :: short
    character(STR_LEN_LONG) :: long,help
    logical :: combWReadSamp,combWSavSamp,combWNoInfc
    integer :: nMax,nMaxScp ! max. # occurrences of option per config file and per command line; … per scope
  end type


  !========================================================[ CONSTANTS ]============================================================

  integer,parameter :: &
   N_CONF_FIL_LINS_MAX=1000, &
   N_CL_OPTS=61

  character(*),parameter :: &
   OPT_PREFX="--", &
   OPT_DESC_SEP=" -+", &
   CL_OPT_LIST_SEP=","

  type(tyCLOptInt),parameter :: &
   I_CL_OPTS=tyCLOptInt(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38, &
   39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61)

  type(tyCLOpt),parameter :: &
   CL_OPTS(N_CL_OPTS)=[ &
   tyCLOpt("calc-hpdi","calc-hpdi=<val>{,<val>{...}}", &
   "Set probability contents of additional HPDI(s) to be calculated for all parameters and derived quantities",.TRUE.,.TRUE., &
   .FALSE.,1,0), &
   tyCLOpt("calc-prob","calc-prob[<par>]=[<lower>,<upper>]", &
   "Calculate posterior probability of <par> lying in the given interval",.TRUE.,.TRUE.,.FALSE.,VAL_UNDEF_INT,VAL_UNDEF_INT), &
   tyCLOpt("calc-prob-all-pars","calc-prob-all-pars={<bounds>|,},{<bounds>|,}...", &
   "Calculate posterior probability of all parameters lying in the given hypercube (<bounds> stands for <lower>,<upper>); omitting &
   &any <bounds> implies that the corresponding parameter is marginalised over",.TRUE.,.TRUE.,.FALSE.,VAL_UNDEF_INT, &
   VAL_UNDEF_INT), &
   tyCLOpt("calc-transit-after","calc-transit-after=<time>","Calculate first primary and secondary transit times after given <&
   &time> (in days, using the same potential time offset as the data); otherwise, if calculated, it refers to the end of the data &
   &timespan",.TRUE.,.TRUE.,.FALSE.,1,0), &
   tyCLOpt("comment","comment=<text>","Include a comment in the log",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("data-dir","data-dir=<dir>","Set data directory",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("discard-frac","discard-frac=<val>", &
   "Set maximum runtime fraction during which to initially discard samples before starting burn-in; if tempering parameters are &
   &adjusted, burn-in is started after 5 minutes at the latest; default: 10%",.FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("discard-psr-scale", &
   "discard-psr-scale=<val>","Set factor by which the difference <PSR>-<PSR threshold> has to decrease for all parameters &
   &before the initial discarding of samples is stopped and burn-in is started, where <PSR> is the PSR value of a &
   &parameter (reference value: maximum PSR attained initially by any parameter) and <PSR threshold> is the threshold for &
   &convergence; this may be overridden if tempering parameters are adjusted (see above) or by maximum runtime fraction for &
   &discarding; default: 0.9",.FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("epoch","epoch=<time>","Set epoch (applies to both prior and posterior)",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("generate-data","generate-data[<type>,{<commands>,}{<# planets>,}<pars>,{<star mass>,}<# data>,<t_1>,<t_N>,&
   &<uncertainty>{,<uncertainty>{...}}]","Generate data of <type>, executing data-related <commands>, for given number of planets &
   &(except for binary-mode data), parameters, star mass (except for binary-mode data), number of data, earliest and latest data &
   &timestamps and measurement uncertainties (for angular-position data: pairs of semi-major and -minor axes of uncertainty &
   &ellipses); {<commands>} stands for {<commands>{,<commands>{...}}} or {$coord_sys=polar|$n_b_obs=2|$with_ang_pos|$with_RV|&
   &$unif_time|$window=[<period>,<fraction>]}, implying respectively a polar coordinate system for &
   &angular-position data, binary-mode RV data, using set of parameters as if angular-position data were included, using set of &
   &parameters as if RV data were included, uniform time sampling, and adding a window defined by period and 1st fraction of &
   &period to be allowed (with 1st period starting at <t_1> and negative <fraction> counting from end of period)",.FALSE.,.TRUE., &
   .FALSE.,1,0), &
   tyCLOpt("group","group[<threshold>]","Modify preceding data set by arranging the data in as few groups as possible &
   &without exceeding <threshold> in any group; with astrometry (angular positions), <threshold> &
   &stands for <time range>,<phi range> (<time range> being the maximum time range [d] per group and <phi range> the max. range &
   &of uncertainty-ellipse rotation angle phi); &
   &with radial velocities, <threshold> stands for <time range>; with astrometry (Hipparcos new-reduction abscissa residuals), &
   &<threshold> stands for &
   &<time range>,<psi range>,<gamma range> (<psi range> being the max. range of the scan-orientation angle and <gamma range> &
   &the max. range of parallax factor divided by its group mean)",.TRUE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,0), &
   tyCLOpt("help","help","Display this help screen",.FALSE.,.FALSE.,.FALSE.,VAL_UNDEF_INT,0), &
   tyCLOpt("help-pars","help-pars","Display information on parameters and derived quantities",.FALSE.,.FALSE.,.FALSE., &
   VAL_UNDEF_INT,0), &
   tyCLOpt("inference-len-frac","inference-len-frac=<val>","Set (minimum) fraction (# links for inference)/(chain length)",.TRUE., &
   .TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("inference-len","inference-len={max|<val>}", &
   "Set # links used for inference to 'maximum' or value (as minimum), where 'maximum' implies that BASE will not stop before &
   &either the maximum chain length is reached or both maximum runtime is reached and burn-in is finished",.FALSE.,.TRUE., &
   .TRUE.,VAL_UNDEF_INT,1),&
   tyCLOpt("K_max-from-vel","K_max-from-vel","Determine maximum value of K from velocity range and minimum eccentricity", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("keep-seed","keep-seed","Reuse last random seed",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("M","M=<val>","Set star mass",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("mail-when-finished","mail-when-finished=<email address>","Send an email to the specified address when finished", &
   .TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("max-dur","max-dur=<val><unit>","Set maximum runtime to be taken by the multi-PT procedure", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("max-len","max-len=<val>","Set maximum length of the chain produced by the multi-PT procedure", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("msini_max","msini_max=<val>","Set maximum value of m*sin(i)",.FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("n-periods-range","n-periods-range=[<lower>,<upper>]","Set assumed range of # periods covered by any data file", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("n-pl","n-pl=<val>{,<val>{...}}","Set number(s) of planets to be assumed",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("n-pt","n-pt=<val>","Set number of PT procedures to be run",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("no-deriv","no-deriv","Disable calculation of derived quantities",.TRUE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("no-files","no-files","Disable any file output",.TRUE.,.FALSE.,.TRUE.,1,0), &
   tyCLOpt("no-gnuplot-exec","no-gnuplot-exec","Don't execute Gnuplot after finishing",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("no-inference","no-inference","Disable inference, i.e. only collect posterior samples",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("no-joint-marg-post","no-joint-marg-post","Don't plot joint marginal posteriors",.TRUE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("no-marg-post","no-marg-post","Don't plot marginal posteriors (also prevents calculation of marginal-posterior modes)", &
   .TRUE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("no-plots","no-plots","Don't produce files for plotting (also prevents calculation of marginal-posterior modes)", &
   .TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("no-ps4pdf-exec","no-ps4pdf-exec","Don't execute ps4pdf after finishing (with PSTricks-compatible plots)", &
   .TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("no-temp-pars-adj","no-temp-pars-adj","Disable adjustment of tempering parameters",.FALSE.,.TRUE.,.TRUE., &
    VAL_UNDEF_INT,1), &
   tyCLOpt("only-plot-data","only-plot-data","Only plot the given data and exit",.FALSE.,.FALSE.,.TRUE.,1,0), &
   tyCLOpt("only-speed-meas","only-speed-meas","Only perform a speed measurement and exit",.FALSE.,.FALSE.,.TRUE.,1,0), &
   tyCLOpt("out-dir","out-dir=<output dir>","Set output directory; default: ../out",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("out-dir-csv","out-dir-csv=<CSV output dir>","Set CSV output directory; default: <output dir>/csv",.TRUE., &
   .TRUE.,.TRUE.,1,0), &
   tyCLOpt("periodogram","periodogram","Only make inference about orbital frequency f (with refined kernel window width)", &
   .TRUE.,.TRUE.,.FALSE.,1,0), &
   tyCLOpt("predict","predict=[<t_1>,<t_n>]", &
   "Predict observables between <t_1> and <t_n> (in days, using the same potential time offset as the data)",.TRUE.,.TRUE., &
   .FALSE.,1,0), &
   tyCLOpt("prior","prior[<par>]=<file>","Get prior for <par> from CSV <file>",.FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT, &
   VAL_UNDEF_INT), &
   tyCLOpt("psr-conv","psr-conv=<val>","Set PSR threshold for convergence",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("pstricks-plots","pstricks-plots","Produce PSTricks-compatible plots",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("pt-n-chains","pt-n-chains=<val>","Set number of chains run by each PT procedure",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("quick-plots","quick-plots","Produce plots of reduced resolution",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("quiet","quiet","Suppress all output to stdout",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("range","range[<par>]={<type>:}[<lower>,<upper>]", &
   "Set prior range of a model parameter; <type> may be {u|j|m|s|n[<n>]}, standing for uniform, Jeffreys, modified Jeffreys, signed&
   & modified Jeffreys and truncated normal prior, respectively; in the latter case, <n> is the ratio of (<upper>-<lower>)/2 to&
   & the standard deviation, and the distribution mean is set to the centre of the interval",.FALSE.,.TRUE., &
   .TRUE.,VAL_UNDEF_INT,VAL_UNDEF_INT), &
   tyCLOpt("read-samples","read-samples=<file>","Read MCMC samples from <file>",.TRUE.,.FALSE.,.TRUE.,1,0), &
   tyCLOpt("save-cand","save-cand","Save the sampled candidates",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("save-samples","save-samples","Save the final MCMC samples",.FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("scatter-plots","scatter-plots","Produce scatter plots",.TRUE.,.TRUE.,.FALSE.,VAL_UNDEF_INT,1), &
   tyCLOpt("scope","scope=<n>","Begin a new scope: the following options will refer to the mode with the <n>th number of planets", &
   .TRUE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,VAL_UNDEF_INT), &
   tyCLOpt("single-plots","single-plots","Output one graphic file per plot",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("speed-meas-dur","speed-meas-dur=<val><unit>","Set maximum duration of speed measurement",.FALSE.,.TRUE.,.TRUE., &
   VAL_UNDEF_INT,1), &
   tyCLOpt("stdout-line-len","stdout-line-len=<val>","Set line length of stdout",.TRUE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("swap","swap=<val>","Set mean spacing between swap proposals; default: 100",.FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("temp-pars","temp-pars=<val>{,<val>{...}}","Set tempering parameters of the heated chains", &
   .FALSE.,.TRUE.,.TRUE.,1,0), &
   tyCLOpt("temp-pars-adj-dur","temp-pars-adj-dur=<val><unit>","Set maximum duration of tempering-parameters adjustment", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,1), &
   tyCLOpt("thin","thin={auto|<val>}","Set stride for thinning to 'automatic' or value (1: no thinning)",.TRUE.,.TRUE.,.TRUE., &
   VAL_UNDEF_INT,1), &
   tyCLOpt("value","value[<par>]=<val>","Set model parameter to a fixed value (equivalent to a delta-function prior)", &
   .FALSE.,.TRUE.,.TRUE.,VAL_UNDEF_INT,VAL_UNDEF_INT), &
   tyCLOpt("zoom-all-pars","zoom-all-pars={<bounds>|,},{<bounds>|,}...", &
   "Set ranges of zoomed (joint) marginal posteriors for all parameters (<bounds> stands for <lower>,<upper>) and only make zoomed &
   &variants of these plots; marginal posteriors of derived quantities are zoomed accordingly; omitting any <bounds> implies that &
   &the prior range will be used for the corresponding parameter",.TRUE.,.TRUE.,.FALSE.,VAL_UNDEF_INT,1)]


  !========================================================[ VARIABLES ]============================================================

  integer(i2b) :: &
   nConfFilLins

  real(dp) :: &
   transTmStart

  logical :: &
   pltPSTricks, &
   confFilExists

  character(STR_LEN_LONG) :: &
   usrComment

  character(MAX_LEN_PATH) :: &
   filNmConf, &
   confFilLins(N_CONF_FIL_LINS_MAX)

  character(STR_LEN_LONGEST) :: &
   cmdLin


!––––––––––––––––––––––––––––––––––––––––––––––––––––––––––[ INTERFACES ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  interface tryOptAssign
    module procedure tryOpt1IntAssign
    module procedure tryOpt1RealAssign
    module procedure tryOpt1DblAssign
    module procedure tryOptIntAssign
    module procedure tryOptRealAssign
    module procedure tryOptDblAssign
    module procedure tryOptStrAssign
  end interface


!–––––––––––––––––––––––––––––––––––––––––––––––––––––––[ SUBPROGRAM PART ]–––––––––––––––––––––––––––––––––––––––––––––––––––––––––

  contains


!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function getOptIndx(optStr,fil)
!
!   Returns:                         index of a user option in CL_OPTS array
!
!   Arguments:
!     optStr                         string containing the option
!     fil                            whether the string is to be interpreted as compatible with config file
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     getOptIndx, &
     i
    logical :: &
     fil
    character(*) :: &
     optStr
    intent(in) :: &
     optStr, &
     fil

    call undef(getOptIndx)

    do i=1,N_CL_OPTS
      if (beginsWith(optStr,trim(mrg("",OPT_PREFX,fil))//trim(CL_OPTS(i)%short))) then
        getOptIndx=i
        exit
      end if
    end do

  end function getOptIndx




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOptCmd(fil,str,cmd)
!
!   Returns:                         whether a string corresponds to a certain command-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     optStr                         string containing the option
!     cmd                            string containing the command in question
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    logical :: &
     fil, &
     tryOptCmd
    character(*) :: &
     str, &
     cmd
    intent(in) :: &
     fil, &
     str, &
     cmd

    tryOptCmd=trim(str) == trim(mrg("",OPT_PREFX,fil))//cmd

  end function tryOptCmd




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOpt1IntAssign(fil,str,ident,val,time)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     val                            assigned value, if found
!     time                           whether value should be interpreted as time value in a given unit
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     val, &
     valArr(:), &
     l, &
     pos1, &
     pos2, &
     nVal
    logical :: &
     fil, &
     tryOpt1IntAssign, &
     time, &
     tm
    character :: &
     un
    character(*) :: &
     str, &
     ident
    allocatable :: &
     valArr
    optional :: &
     time
    intent(in) :: &
     fil, &
     str, &
     ident, &
     time
    intent(inout) :: &
     val

    tm=present(time) .AND. time
    l=len_trim(str)
    if (tm) then
      pos2=l-1
      un=str(l:l)
    else
      pos2=l
    end if

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      if (tm .AND. isUndef(posInArr(["h","m","s"],un))) call err("assignment of '"//ident//"': wrong format")
      call xplode(str(pos1:pos2),CL_OPT_LIST_SEP,valArr)
      nVal=size(valArr,1)
      if (nVal /= 1) call err("assignment of '"//ident//"': wrong format")
      val=valArr(1)
      if (tm) val=val*merge(60,merge(3600,1,un == "h"),un == "m")
      tryOpt1IntAssign=.TRUE.
    else
      tryOpt1IntAssign=.FALSE.
    end if

    return

  end function tryOpt1IntAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOpt1RealAssign(fil,str,ident,val,time)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     val                            assigned value, if found
!     time                           whether value should be interpreted as time value in a given unit
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     pos1, &
     pos2, &
     nVal, &
     l
    real :: &
     val, &
     valArr(:)
    logical :: &
     fil, &
     tryOpt1RealAssign, &
     time, &
     tm
    character :: &
     un
    character(*) :: &
     str, &
     ident
    allocatable :: &
     valArr
    optional :: &
     time
    intent(in) :: &
     fil, &
     str, &
     ident, &
     time
    intent(inout) :: &
     val

    tm=present(time) .AND. time
    l=len_trim(str)
    if (tm) then
      pos2=l-1
      un=str(l:l)
    else
      pos2=l
    end if

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      if (tm .AND. isUndef(posInArr(["h","m","s"],un))) call err("assignment of '"//ident//"': wrong format")
      call xplode(str(pos1:pos2),CL_OPT_LIST_SEP,valArr)
      nVal=size(valArr,1)
      if (nVal /= 1) call err("assignment of '"//ident//"': wrong format")
      val=valArr(1)
      if (tm) val=val*merge(60,merge(3600,1,un == "h"),un == "m")
      tryOpt1RealAssign=.TRUE.
    else
      tryOpt1RealAssign=.FALSE.
    end if

    return

  end function tryOpt1RealAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOpt1DblAssign(fil,str,ident,val,time)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     val                            assigned value, if found
!     time                           whether value should be interpreted as time value in a given unit
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     pos1, &
     pos2, &
     nVal, &
     l
    real(dp) :: &
     val, &
     valArr(:)
    logical :: &
     fil, &
     tryOpt1DblAssign, &
     time, &
     tm
    character :: &
     un
    character(*) :: &
     str, &
     ident
    allocatable :: &
     valArr
    optional :: &
     time
    intent(in) :: &
     fil, &
     str, &
     ident, &
     time
    intent(inout) :: &
     val

    tm=present(time) .AND. time
    l=len_trim(str)
    if (tm) then
      pos2=l-1
      un=str(l:l)
    else
      pos2=l
    end if

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      if (tm .AND. isUndef(posInArr(["h","m","s"],un))) call err("assignment of '"//ident//"': wrong format")
      call xplode(str(pos1:pos2),CL_OPT_LIST_SEP,valArr)
      nVal=size(valArr,1)
      if (nVal /= 1) call err("assignment of '"//ident//"': wrong format")
      val=valArr(1)
      if (tm) val=val*merge(60,merge(3600,1,un == "h"),un == "m")
      tryOpt1DblAssign=.TRUE.
    else
      tryOpt1DblAssign=.FALSE.
    end if

    return

  end function tryOpt1DblAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOptIntAssign(fil,str,ident,vals)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     vals                           assigned values, if found
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     vals(:), &
     nReq, &
     nVal, &
     pos1
    logical :: &
     fil, &
     tryOptIntAssign
    character(*) :: &
     str, &
     ident
    allocatable :: &
     vals
    intent(in) :: &
     fil, &
     str, &
     ident
    intent(inout) :: &
     vals

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      nReq=size(vals)
      call xplode(rmLead(rmTrail(trim(str(pos1:)),"]"),"["),CL_OPT_LIST_SEP,vals)
      nVal=size(vals,1)
      if (nVal < nReq) call err("assignment of '"//ident//"': wrong format")
      tryOptIntAssign=.TRUE.
    else
      tryOptIntAssign=.FALSE.
    end if

  end function tryOptIntAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOptRealAssign(fil,str,ident,vals)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     vals                           assigned values, if found
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     nReq, &
     nVal, &
     pos1
    real :: &
     vals(:)
    logical :: &
     fil, &
     tryOptRealAssign
    character(*) :: &
     str, &
     ident
    allocatable :: &
     vals
    intent(in) :: &
     fil, &
     str, &
     ident
    intent(inout) :: &
     vals

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      nReq=size(vals)
      call xplode(rmLead(rmTrail(trim(str(pos1:)),"]"),"["),CL_OPT_LIST_SEP,vals)
      nVal=size(vals,1)
      if (nVal < nReq) call err("assignment of '"//ident//"': wrong format")
      tryOptRealAssign=.TRUE.
    else
      tryOptRealAssign=.FALSE.
    end if

  end function tryOptRealAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOptDblAssign(fil,str,ident,vals)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     vals                           assigned values, if found
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     nReq, &
     nVal, &
     pos1
    real(dp) :: &
     vals(:)
    logical :: &
     fil, &
     tryOptDblAssign
    character(*) :: &
     str, &
     ident
    allocatable :: &
     vals
    intent(in) :: &
     fil, &
     str, &
     ident
    intent(inout) :: &
     vals

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      nReq=size(vals)
      call xplode(rmLead(rmTrail(trim(str(pos1:)),"]"),"["),CL_OPT_LIST_SEP,vals)
      nVal=size(vals,1)
      if (nVal < nReq) call err("assignment of '"//ident//"': wrong format")
      tryOptDblAssign=.TRUE.
    else
      tryOptDblAssign=.FALSE.
    end if

  end function tryOptDblAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    function tryOptStrAssign(fil,str,ident,val)
!
!   Returns:                         whether a string corresponds to a certain assignment-type user option
!
!   Arguments:
!     fil                            whether the string is to be interpreted as compatible with config file
!     str                            string containing the option
!     ident                          string containing the part of the assignment in question left of the = sign
!     val                            assigned value, if found
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none

    integer :: &
     pos1
    logical :: &
     fil, &
     tryOptStrAssign
    character(*) :: &
     str, &
     ident, &
     val
    intent(in) :: &
     fil, &
     str, &
     ident
    intent(inout) :: &
     val

    if (beginsWith(str,trim(mrg("",OPT_PREFX,fil))//ident//"=",pos1,OPT_LONGER)) then
      if (pos1 == 1) goto 9991
      read(str(pos1:),"(A)",err=9991,end=9991) val
      tryOptStrAssign=.TRUE.
    else
      tryOptStrAssign=.FALSE.
    end if

    return

9991 call err("assignment of '"//ident//"': wrong format")

  end function tryOptStrAssign




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine printHelp
!
!   prints a help screen including all user options
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     i

    usrQuiet=.FALSE.                 ! 'quiet' option could be, e.g., in base.conf, hindering user from getting help


    ! header

    call printMsg(UNIT%cons,"BASE - "//BASE_LONG//trim(niceFmt(BASE_REV_NR)))
    call printMsg(UNIT%cons,uCaseFst(AUTHOR_STR))
    call printMsg(UNIT%cons,"")

    call startSubsect(UNIT%cons,"Usage:",spcPrefx=.TRUE.)
    call printMsg(UNIT%cons,"base {<options>} <data file> {<options>|<data file> {...}}",setFlg(FLG%spcPrfx))
    call endSubsect(UNIT%cons,spcPrefx=.TRUE.) ! Usage
    call startSubsect(UNIT%cons,"Notation:",spcPrefx=.TRUE.)
    call printOption("{<expression>}","Optional expression")
    call printOption("{<expression>|<expression>|...}","List of alternative expressions")
    call printOption("{...}","Recursive insertion of enclosing optional expression")
    call printOption("...","Repetition of expression from and including the previous | or ,")
    call printMsg(UNIT%cons,"",setFlg(FLG%spcPrfx))
    call endSubsect(UNIT%cons,spcPrefx=.TRUE.) ! notation


    ! options

    call startSubsect(UNIT%cons,"Options:",spcPrefx=.TRUE.)
    call printMsg(UNIT%cons,"(Note that options can be given on the command line (preceded by "//OPT_PREFX// &
     ") or in the config file "//STD_FIL_NM_CONF//".)",setFlg(FLG%spcPrfx))
    call printMsg(UNIT%cons,"",setFlg(FLG%spcPrfx))

    do i=1,N_CL_OPTS
      call printOption(CL_OPTS(i)%long,CL_OPTS(i)%help)
    end do
    
    call endSubsect(UNIT%cons,spcPrefx=.TRUE.) ! Options


  end subroutine printHelp




!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!
    subroutine printOption(opt,desc)
!
!   prints a user option and its description
!
!   Arguments:
!     opt                            string containing the option
!     desc                           string containing a description of the option
!
!
!:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    implicit none
    integer :: &
     optLen, &
     descLen, &
     pos1, &
     pos2, &
     pos3, &
     pos4, &
     descLenMax, &
     descChunkLen, &
     descPos
    logical :: &
     indentLongOpt
    character(*) :: &
     opt, &
     desc
    character(len_trim(opt)) :: &
     trimOpt
    character(len_trim(desc)) :: &
     trimDesc
    intent(in) :: &
     opt, &
     desc


    if (.NOT. usrQuiet) then

      trimOpt=trim(opt)
      optLen=len(trimOpt)
      trimDesc=trim(desc)
      descLen=len(trimDesc)

      pos1=1+sectDepth(I_UNIT%cons)*CONS_TAB_WIDTH
      pos2=pos1+optLen
      pos3=consTabStopOpt
      pos4=consLineLen+1
      descLenMax=pos4-pos3

      indentLongOpt=.FALSE.

      ! print option

      write(*,"(A)",advance="no") indentPrefx(I_UNIT%cons,spc=.TRUE.)
      if (pos2+OPT_DESC_SPC > pos3) then
        ! long option
        write(*,"(A)") trimOpt
        write(*,"(A)",advance="no") repeat(" ",pos3-1)
      else
        write(*,"(A)",advance="no") trimOpt//repeat(" ",pos3-pos2)
      end if


      ! print description

      if (descLen > descLenMax) then
        ! long description
        descPos=1
        do
          descChunkLen=nxtChunkLen()
          if (descChunkLen <= 0) exit
          if (indentLongOpt) write(*,"(A)",advance="no") repeat(" ",pos3-1)
          write(*,"(A)") trimDesc(descPos:descPos+descChunkLen-1)
          descPos=descPos+descChunkLen
          indentLongOpt=.TRUE.
        end do
      else
        write(*,"(A)") trimDesc
      end if

    end if


    contains

    function nxtChunkLen()           ! returns length of next chunk to be printed

      implicit none
      integer :: &
       nxtChunkLen, &
       sepPos, &
       tmpSepPos, &
       tmpPos1


      ! remove spaces at the beginning of the current chunk of description
      do
        if (trimDesc(descPos:descPos) /= " ") exit
        descPos=descPos+1
      end do


      if (descLen+1-descPos <= descLenMax) then

        ! we can complete the description now
        nxtChunkLen=descLen+1-descPos

      else

        ! find next separator; otherwise separate at any position, maximising current chunk len.

        call undef(sepPos)
        tmpPos1=descPos
        do
          tmpSepPos=scan(trimDesc(tmpPos1:),OPT_DESC_SEP)+tmpPos1-1
          if (tmpSepPos == tmpPos1-1 .OR. tmpSepPos-descPos > descLenMax) exit
          sepPos=tmpSepPos
          tmpPos1=tmpSepPos+1
          if (tmpPos1 > descLen) exit
        end do

        if (isUndef(sepPos)) then
          ! sep. not found
          nxtChunkLen=min(descLenMax,descLen-(descPos-1))
        else
          nxtChunkLen=min(descLenMax,sepPos-descPos+1) ! including sep.
        end if

      end if

    end function

  end subroutine printOption




end module
